<?php
/*
 * Copyright (C) 2017 Manuela v.d.Decken <manuela@isteam.de>
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT OR THIS HEADER
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License 2 for more details.
 *
 * You should have received a copy of the GNU General Public License 2
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Description of detail
 *
 * @package      Core package
 * @copyright    Manuela v.d.Decken <manuela@isteam.de>
 * @author       Manuela v.d.Decken <manuela@isteam.de>
 * @license      GNU General Public License 2.0
 * @version      0.0.1
 * @revision     $Id: details.php 66 2018-09-17 15:48:07Z Luisehahne $
 * @since        File available since 04.11.2017
 * @deprecated   no / since 0000/00/00
 * @description  xxx
 */
declare(strict_types = 1);
//declare(encoding = 'UTF-8');

use bin\{WbAdaptor,SecureTokens,Sanitize};
use bin\helpers\{PreCheck};
use vendor\phplib\Template;

// Include config file and admin class file
    if (!defined('SYSTEM_RUN') ){ require  dirname(__DIR__,2).'/config.php'; }
/* -------------------------------------------------------- */
// register addon vars
    $aAddon       = [];
    $sAddonType   =  'template';
    $sAddonAppDir = '/templates/';
    $sAddonDir    = ltrim($sAddonAppDir,'/');
    $sAddonDirectory = '';
    $aDebug = [];
    $aRequestVars = [];
    $aPreventFromUninstall = [
        'DefaultTheme',
        'DefaultTemplate',
//        'ccc',
//        'ddd',
    ];

    $admin = new admin('Addons', $sAddonType.'s_view', true);
/* -------------------------------------------------------- */
    $oReg     = WbAdaptor::getInstance();
    $oRequest = $oReg->getRequester();
    $aVarnames = $oRequest->getParamNames();
    foreach ($aVarnames as $sName) {
        $aRequestVars[$sName] = $oReg->Request->getParam($sName);
    }
/* -------------------------------------------------------- */
    $oApp     = $oReg->getApplication();
    $oDb      = $oReg->getDatabase();
    $oTrans   = $oReg->getTranslate();
    $sDomain  = $oApp->getDirNamespace($oReg->AcpPath.'addons');
    $oTrans->enableAddon($sDomain);
    $aTrans = $oTrans->getLangArray();

try {

// get request method
    $oRequests = (object) filter_input_array(
        (strtoupper($_SERVER['REQUEST_METHOD']) === 'POST' ? INPUT_POST : INPUT_GET),
        FILTER_UNSAFE_RAW
    );

// needed to set the advanced block
    $show_block = isset($oRequests->advanced) && (int)$oRequests->advanced;
    $sAddonBackUrl  = $oReg->AcpUrl.basename(__DIR__).'/index.php'.($show_block ? '?advanced='.$show_block : '');
    if (!\bin\SecureTokens::checkFTAN()){
        throw new \Exception($oTrans->MESSAGE_GENERIC_SECURITY_ACCESS);
    }

    if (!isset($oRequests->file) || $oRequests->file == false) {
        throw new \Exception($oTrans->MESSAGE_GENERIC_FORGOT_OPTIONS);
    } else {
        $iAddonId = \bin\SecureTokens::checkIDKEY($oRequests->file);
    }
    if ($iAddonId === 0){
        throw new \Exception($oTrans->MESSAGE_GENERIC_SECURITY_ACCESS);
    }

// Get addon directory
    $sqlAddons = 'SELECT `directory` FROM `'.$oReg->TablePrefix.'addons` '
               . 'WHERE `addon_id`='.(int)$iAddonId.' ';
    if (($sAddonDirectory = $database->get_one($sqlAddons))) {
        $sAddonDirectory = preg_replace('/[^a-z0-9_-]/i', "", $sAddonDirectory);  // fix secunia 2010-92-2
    }
    $sAddonInfoFile = $oReg->AppPath.$sAddonDir.$sAddonDirectory.'/info.php';
    $sTplDomain  = $oApp->getDirNamespace($oReg->AppPath.$sAddonDir.$sAddonDirectory);
    $oTrans->enableAddon($sTplDomain);
/*
print '<pre  class="mod-pre" style="margin-left:30px;">function <span>'.__FUNCTION__.'( '.''.' );</span>  filename: <span>'.basename(__FILE__).'</span>  line: '.__LINE__.' -> '."\n";
print_r( $sTplDomain ); print '</pre>'; \flush (); //  sleep(10); die();
*/
// Check if the add-on exists
    if (!file_exists($oReg->AppPath.$sAddonDir.$sAddonDirectory.'/info.php')) {
        throw new \Exception($sAddonDir.$sAddonDirectory."\n".$oTrans->MESSAGE_GENERIC_NOT_INSTALLED);
    }

// Setup template object, parse vars to it, then parse it
// Create new template object
    $template = new Template(dirname($admin->correct_theme_source($sAddonType.'s_details.htt')));
// $template->debug = true;
    $template->set_file('page', $sAddonType.'s_details.htt');
    $template->set_block('page', 'main_block', 'main');

// Insert values
    $sql = 'SELECT * FROM `'.$oReg->TablePrefix.'addons` '
    .'WHERE `type` = \''.$sAddonType.'\' '
    .'AND `directory` = \''.$sAddonDirectory.'\'';
    if ($oAddon = $database->query($sql)) {
        $aAddon = $oAddon->fetchAssoc();
    }

// check if a addon description exists for the displayed backend language
    $tool_description = false;
    if (function_exists('file_get_contents') && file_exists($oReg->AppPath.$sAddonDir.$sAddonDirectory.'languages/'.LANGUAGE .'.php')) {
        // read contents of the addon language file into string
        $data = file_get_contents($oReg->AppPath.$sAddonDir.$sAddonDirectory.'languages/' .LANGUAGE .'.php');
        // use regular expressions to fetch the content of the variable from the string
        $tool_description = get_variable_content($sAddonType.'_description', $data, false, false);
        // replace optional placeholder {WB_URL} with value stored in config.php
        if ($tool_description !== false && strlen(trim($tool_description)) != 0) {
            $tool_description = str_replace('{WB_URL}', $oReg->AppUrl, $tool_description);
        } else {
            $tool_description = false;
        }
    }

   if($tool_description !== false) {
        // Override the addon-description with correct desription in users language
        $aAddon['description'] = $tool_description;
    }

    $template->set_var(array(
            'ADMIN_URL' => $oReg->AcpUrl,
            'WB_URL' => $oReg->AppUrl,
            'THEME_URL' => $oReg->ThemeUrl,
            'AddonBackUrl' => $sAddonBackUrl,
          )
    );

    $sType = ($aAddon['function'] ?: $aAddon['type']);
    $sTypeMsg = $oTrans->{'TEXT_' . strtoupper($sType)};

    $template->set_var(array(
              'NAME' => $aAddon['name'],
              'TYPE' => $sTypeMsg,
              'AUTHOR' => $aAddon['author'],
              'DESCRIPTION' => $aAddon['description'],
              'VERSION' => $aAddon['version'],
              'DESIGNED_FOR' => $aAddon['platform'],
          )
    );

// Insert language text and messages
    $template->set_var($aTrans);
// Parse addon object
    $template->parse('main', 'main_block', false);
    $template->pparse('output', 'page');

}catch (\Exception $ex) {
    $sErrMsg = PreCheck::xnl2br(sprintf('[%d] %s', $ex->getLine(), $ex->getMessage()));
    $admin->print_error ($sErrMsg, $sAddonBackUrl);
    exit;
}

    $oTrans->disableAddon();

// Print admin footer
$admin->print_footer();
