/* jspanel.js - License MIT, copyright 2013 - 2018 Stefan Straesser <info@jspanel.de> (https://jspanel.de) */
/* global jsPanel, $ */
'use strict';

export const jsPanel = {

    version: '4.2.1',
    date:    '2018-09-14 13:11',
    ajaxAlwaysCallbacks: [],
    autopositionSpacing: 4,
    closeOnEscape: (() => {
        document.addEventListener('keydown', (e) => {
            if (e.key === 'Escape' || e.code === 'Esc' || e.keyCode === 27) {
                jsPanel.getPanels(function() {
                    return this.classList.contains('jsPanel');
                }).some((item) => {
                    if (item.options.closeOnEscape) {
                        item.close();
                        return true;
                    }
                    return false;
                });
            }
        },false);
    })(),
    defaults: {
        boxShadow: 3,
        container: document.body,
        contentSize: {width: '400px', height: '200px'}, // must be object
        dragit: {
            cursor:  'move',
            handles: '.jsPanel-headerlogo, .jsPanel-titlebar, .jsPanel-ftr', // do not use .jsPanel-headerbar
            opacity: 0.8,
            disableOnMaximized: true
        },
        header: true,
        headerTitle: 'jsPanel',
        headerControls: 'all',
        iconfont: false,
        maximizedMargin: 0,
        minimizeTo: 'default',
        paneltype: 'standard',
        position: 'center',
        resizeit: {
            handles:     'n, e, s, w, ne, se, sw, nw',
            minWidth:    128,
            minHeight:   128,
        },
        theme: 'default'
    },
    defaultSnapConfig: {
        sensitivity: 70,
        trigger: 'panel'
    },
    error: (() => {
        // Create a new object, that prototypically inherits from the Error constructor
        if (!window.jsPanelError) {
            window.jsPanelError = function (message) {
                this.name = 'jsPanelError';
                this.message = message || '';
                this.stack = (new Error()).stack;
            };
            window.jsPanelError.prototype = Object.create(Error.prototype);
            window.jsPanelError.prototype.constructor = window.jsPanelError;
        }
    })(),
    extensions: {},
    globalCallbacks: false,
    icons: {
        close:       `<svg class="jsPanel-icon" version="1.1" xmlns="http://www.w3.org/2000/svg" width="20" height="20" viewBox="0 0 28 28"><path fill="currentColor" d="M17.75 16l9.85-9.85c0.5-0.5 0.5-1.3 0-1.75-0.5-0.5-1.3-0.5-1.75 0l-9.85 9.85-9.85-9.9c-0.5-0.5-1.3-0.5-1.75 0-0.5 0.5-0.5 1.3 0 1.75l9.85 9.9-9.9 9.85c-0.5 0.5-0.5 1.3 0 1.75 0.25 0.25 0.55 0.35 0.9 0.35s0.65-0.1 0.9-0.35l9.85-9.85 9.85 9.85c0.25 0.25 0.55 0.35 0.9 0.35s0.65-0.1 0.9-0.35c0.5-0.5 0.5-1.3 0-1.75l-9.9-9.85z"></path></svg>`,
        maximize:    `<svg class="jsPanel-icon" version="1.1" xmlns="http://www.w3.org/2000/svg" width="20" height="20" viewBox="0 0 28 28"><path fill="currentColor" d="M27.55 3.9h-22.6c-0.55 0-1 0.45-1 1v22.3c0 0.55 0.45 1 1 1h22.55c0.55 0 1-0.45 1-1v-22.3c0.050-0.55-0.4-1-0.95-1zM5.95 26.15v-18h20.55v18h-20.55z"></path></svg>`,
        normalize:   `<svg class="jsPanel-icon" version="1.1" xmlns="http://www.w3.org/2000/svg" width="20" height="20" viewBox="0 0 28 28"><path fill="currentColor" d="M27.9 3.75h-18.8c-0.4 0-0.75 0.35-0.75 0.75v4.3c0 0.1 0 0.2 0.050 0.3h-4.2c-0.55 0-1 0.45-1 1v17.4c0 0.55 0.45 1 1 1h17.65c0.55 0 1-0.45 1-1v-3.7c0.050 0 0.1 0.050 0.2 0.050h4.9c0.4 0 0.75-0.35 0.75-0.75v-18.6c-0.050-0.4-0.4-0.75-0.8-0.75zM5.2 26.5v-12.95c0.050 0 0.1 0 0.15 0h15.4c0.050 0 0.1 0 0.15 0v12.95h-15.7zM27.15 22.35h-4.15c-0.050 0-0.15 0-0.2 0.050v-12.3c0-0.55-0.45-1-1-1h-12c0.050-0.1 0.050-0.2 0.050-0.3v-3.55h17.3v17.1z"></path></svg>`,
        minimize:    `<svg class="jsPanel-icon" version="1.1" xmlns="http://www.w3.org/2000/svg" width="20" height="20" viewBox="0 0 28 28"><path fill="currentColor" d="M27.3 28.5h-22.6c-0.85 0-1.5-0.65-1.5-1.5s0.65-1.5 1.5-1.5h22.55c0.85 0 1.5 0.65 1.5 1.5s-0.65 1.5-1.45 1.5z"></path></svg>`,
        smallifyrev: `<svg class="jsPanel-icon" version="1.1" xmlns="http://www.w3.org/2000/svg" width="20" height="20" viewBox="0 0 28 28"><path fill="currentColor" d="M15.95 23.2c0 0 0 0 0 0-0.35 0-0.65-0.15-0.9-0.35l-11.7-11.9c-0.5-0.5-0.5-1.3 0-1.75 0.5-0.5 1.3-0.5 1.75 0l10.85 10.95 10.9-10.8c0.5-0.5 1.3-0.5 1.75 0s0.5 1.3 0 1.75l-11.75 11.7c-0.25 0.25-0.55 0.4-0.9 0.4z"></path></svg>`,
        smallify:    `<svg class="jsPanel-icon" version="1.1" xmlns="http://www.w3.org/2000/svg" width="20" height="20" viewBox="0 0 28 28"><path fill="currentColor" d="M28.65 20.85l-11.8-11.65c-0.5-0.5-1.3-0.5-1.75 0l-11.75 11.85c-0.5 0.5-0.5 1.3 0 1.75 0.25 0.25 0.55 0.35 0.9 0.35 0.3 0 0.65-0.1 0.9-0.35l10.85-10.95 10.9 10.8c0.5 0.5 1.3 0.5 1.75 0 0.5-0.5 0.5-1.3 0-1.8z"></path></svg>`
    },
    idCounter: 0,
    isIE: (() => {return navigator.appVersion.match(/Trident/);})(),
    mdbthemes: ['secondary', 'elegant', 'stylish', 'unique', 'special'],
    pointerdown: ('ontouchend' in window) ? ['touchstart', 'mousedown'] : ['mousedown'],
    pointermove: ('ontouchend' in window) ? ['touchmove', 'mousemove'] : ['mousemove'],
    pointerup:   ('ontouchend' in window) ? ['touchend', 'mouseup'] : ['mouseup'],
    polyfills: (() => {
        // .append() polyfill needed for EDGE - https://developer.mozilla.org/en-US/docs/Web/API/ParentNode/append
        (function (arr) {
            arr.forEach(function (item) {
                item.append = item.append || function () {
                    let argArr = Array.prototype.slice.call(arguments),
                        docFrag = document.createDocumentFragment();
                    argArr.forEach(function (argItem) {
                        let isNode = argItem instanceof Node;
                        docFrag.appendChild(isNode ? argItem : document.createTextNode(String(argItem)));
                    });
                    this.appendChild(docFrag);
                };
            });
        })([Element.prototype, Document.prototype, DocumentFragment.prototype]);
        // Element.closest() polyfill needed for EDGE - https://developer.mozilla.org/en-US/docs/Web/API/Element/closest
        if (window.Element && !Element.prototype.closest) {
            Element.prototype.closest =
                function(s) {
                    let matches = (this.document || this.ownerDocument).querySelectorAll(s),
                        i,
                        el = this;
                    do {
                        i = matches.length;
                        // eslint-disable-next-line no-empty
                        while (--i >= 0 && matches.item(i) !== el) {}
                    } while ((i < 0) && (el = el.parentElement));
                    return el;
                };
        }
        // NodeList.prototype.forEach() polyfill needed for IE11 and Android mobile - https://developer.mozilla.org/en-US/docs/Web/API/NodeList/forEach
        if (window.NodeList && !NodeList.prototype.forEach) {
            NodeList.prototype.forEach = function (callback, thisArg) {
                thisArg = thisArg || window;
                for (let i = 0; i < this.length; i++) {
                    callback.call(thisArg, this[i], i, this);
                }
            };
        }
        // Object.assign Polyfill needed for mobiles - https://developer.mozilla.org/de/docs/Web/JavaScript/Reference/Global_Objects/Object/assign
        if (!Object.assign) {
            Object.defineProperty(Object, 'assign', {
                enumerable:   false,
                configurable: true,
                writable:     true,
                value:        function (target) {
                    if (target === undefined || target === null) {
                        throw new TypeError('Cannot convert first argument to object');
                    }
                    let to = Object(target);
                    for (let i = 1; i < arguments.length; i++) {
                        let nextSource = arguments[i];
                        if (nextSource === undefined || nextSource === null) {
                            continue;
                        }
                        nextSource = Object(nextSource);
                        let keysArray = Object.keys(Object(nextSource));
                        for (let nextIndex = 0, len = keysArray.length; nextIndex < len; nextIndex++) {
                            let nextKey = keysArray[nextIndex];
                            let desc = Object.getOwnPropertyDescriptor(nextSource, nextKey);
                            if (desc !== undefined && desc.enumerable) {
                                to[nextKey] = nextSource[nextKey];
                            }
                        }
                    }
                    return to;
                }
            });
        }
        // Polyfills for IE11 only
        // CustomEvent - https://developer.mozilla.org/en-US/docs/Web/API/CustomEvent
        (function () {
            if ( typeof window.CustomEvent === 'function' ) return false;
            function CustomEvent ( event, params ) {
                params = params || { bubbles: false, cancelable: false, detail: undefined };
                let evt = document.createEvent( 'CustomEvent' );
                evt.initCustomEvent( event, params.bubbles, params.cancelable, params.detail );
                return evt;
            }
            CustomEvent.prototype = window.Event.prototype;
            window.CustomEvent = CustomEvent;
        })();
        // String.prototype.endsWith() - https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/endsWith
        if (!String.prototype.endsWith) {
            String.prototype.endsWith = function (searchStr, Position) {
                // This works much better than >= because
                // it compensates for NaN:
                if (!(Position < this.length))
                    Position = this.length;
                else
                    Position |= 0; // round position
                return this.substr(Position - searchStr.length,
                    searchStr.length) === searchStr;
            };
        }
        // String.prototype.startsWith() - https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/startsWith
        if (!String.prototype.startsWith) {
            String.prototype.startsWith = function(searchString, position){
                return this.substr(position || 0, searchString.length) === searchString;
            };
        }
    })(),
    themes: ['default', 'primary', 'info', 'success', 'warning', 'danger'],
    ziBase: 100,

    addScript(path, type = 'text/javascript', callback) {
        let script = document.createElement('script');
        script.onload = callback;
        script.src = path;
        script.type = type;
        document.head.appendChild(script);
    },

    ajax(obj, ajaxConfig) {
        // check whether obj is a jsPanel or something else
        let objIsPanel;
        if (typeof obj === 'object' && obj.classList.contains('jsPanel')) {
            objIsPanel = true;
        } else {
            objIsPanel = false;
            if (typeof obj === 'string') {
                obj = document.querySelector(obj);
            }
        }

        const conf = ajaxConfig,
            configDefaults = {
                method: 'GET',
                async: true,
                user: '',
                pwd: '',
                done: function () {
                    objIsPanel
                        ? obj.content.innerHTML = this.responseText
                        : obj.innerHTML = this.responseText;
                },
                autoresize: true,
                autoreposition: true
            };
        let config;

        if (typeof conf === 'string') {
            config = Object.assign({}, configDefaults, {url: encodeURI(conf), evalscripttags: true});
        } else if (typeof conf === 'object' && conf.url) {
            config = Object.assign({}, configDefaults, conf);
            config.url = encodeURI(conf.url);
            // reset timeout to 0, withCredentials & responseType to false if request is synchronous
            if (config.async === false) {
                config.timeout = 0;
                if (config.withCredentials) {config.withCredentials = undefined;}
                if (config.responseType) {config.responseType = undefined;}
            }
        } else {
            console.info('XMLHttpRequest seems to miss the request url!');
            return obj;
        }

        const xhr = new XMLHttpRequest();

        xhr.onreadystatechange = () => {
            if (xhr.readyState === 4) {
                if (xhr.status === 200) {
                    config.done.call(xhr, obj);

                    // extract and eval content of script tags if "evalscripttags"
                    if (config.evalscripttags) {
                        // get all script tags within responseText
                        const scripttags = xhr.responseText.match(/<script\b[^>]*>([\s\S]*?)<\/script>/gi);
                        if (scripttags) {
                            scripttags.forEach((tag) => {
                                // remove tags from string and trim it
                                let js = tag.replace(/<script\b[^>]*>/i, '').replace(/<\/script>/i, '').trim();
                                // execute javascript
                                eval(js);
                            });
                        }
                    }

                } else {
                    if (config.fail) {config.fail.call(xhr, obj);}
                }

                if (config.always) {config.always.call(xhr, obj);}

                // resize and reposition panel if either width or height is set to 'auto'
                if (objIsPanel) {
                    const oContentSize = obj.options.contentSize;
                    if (typeof oContentSize === 'string' && oContentSize.match(/auto/i)) {
                        const parts = oContentSize.split(' ');
                        const sizes = Object.assign({}, {width: parts[0], height: parts[1]});
                        if (config.autoresize) {
                            obj.resize(sizes);
                        }
                        if (!obj.classList.contains('jsPanel-contextmenu')) {
                            if (config.autoreposition) {
                                obj.reposition();
                            }
                        }
                    } else if (typeof oContentSize === 'object' && (oContentSize.width === 'auto' || oContentSize.height === 'auto')) {
                        const sizes = Object.assign({}, oContentSize);
                        if (config.autoresize) {
                            obj.resize(sizes);
                        }
                        if (!obj.classList.contains('jsPanel-contextmenu')) {
                            if (config.autoreposition) {
                                obj.reposition();
                            }
                        }
                    }
                }

                // allows plugins to add callback functions to the ajax always callback
                if (jsPanel.ajaxAlwaysCallbacks.length) {
                    jsPanel.ajaxAlwaysCallbacks.forEach((item) => {
                        item.call(obj, obj);
                    });
                }

            }
        };

        xhr.open(config.method, config.url, config.async, config.user, config.pwd);
        xhr.timeout = config.timeout || 0;
        if (config.withCredentials) {xhr.withCredentials = config.withCredentials;}
        if (config.responseType) {xhr.responseType = config.responseType;}

        if (config.beforeSend) {
            config.beforeSend.call(xhr);
        }
        if (config.data) {
            xhr.send(config.data);
        } else {
            xhr.send(null);
        }

        return obj;
    },

    calcColors(primaryColor) {
        const primeColor = this.color(primaryColor),
            secondColor = this.lighten(primaryColor, 0.81),
            thirdColor = this.darken(primaryColor, 0.5),
            fontColorForPrimary = this.perceivedBrightness(primaryColor) <= 0.556 ? '#ffffff' : '#000000',
            fontColorForSecond = this.perceivedBrightness(secondColor) <= 0.556 ? '#ffffff' : '#000000',
            fontColorForThird = this.perceivedBrightness(thirdColor) <= 0.556 ? '#000000' : '#ffffff';
        return [primeColor.hsl.css, secondColor, thirdColor, fontColorForPrimary, fontColorForSecond, fontColorForThird];
    },

    color(val) {

        let color = val.toLowerCase(),
            r, g, b, h, s, l, match, channels, hsl, result = {};
        const hexPattern = /^#?([0-9a-f]{3}|[0-9a-f]{6})$/gi, // matches "#123" or "#f05a78" with or without "#"
            RGBAPattern = /^rgba?\(([0-9]{1,3}),([0-9]{1,3}),([0-9]{1,3}),?(0|1|0\.[0-9]{1,2}|\.[0-9]{1,2})?\)$/gi, // matches rgb/rgba color values, whitespace allowed
            HSLAPattern = /^hsla?\(([0-9]{1,3}),([0-9]{1,3}%),([0-9]{1,3}%),?(0|1|0\.[0-9]{1,2}|\.[0-9]{1,2})?\)$/gi,
            namedColors = {
                aliceblue:            'f0f8ff',
                antiquewhite:         'faebd7',
                aqua:                 '0ff',
                aquamarine:           '7fffd4',
                azure:                'f0ffff',
                beige:                'f5f5dc',
                bisque:               'ffe4c4',
                black:                '000',
                blanchedalmond:       'ffebcd',
                blue:                 '00f',
                blueviolet:           '8a2be2',
                brown:                'a52a2a',
                burlywood:            'deb887',
                cadetblue:            '5f9ea0',
                chartreuse:           '7fff00',
                chocolate:            'd2691e',
                coral:                'ff7f50',
                cornflowerblue:       '6495ed',
                cornsilk:             'fff8dc',
                crimson:              'dc143c',
                cyan:                 '0ff',
                darkblue:             '00008b',
                darkcyan:             '008b8b',
                darkgoldenrod:        'b8860b',
                darkgray:             'a9a9a9',
                darkgrey:             'a9a9a9',
                darkgreen:            '006400',
                darkkhaki:            'bdb76b',
                darkmagenta:          '8b008b',
                darkolivegreen:       '556b2f',
                darkorange:           'ff8c00',
                darkorchid:           '9932cc',
                darkred:              '8b0000',
                darksalmon:           'e9967a',
                darkseagreen:         '8fbc8f',
                darkslateblue:        '483d8b',
                darkslategray:        '2f4f4f',
                darkslategrey:        '2f4f4f',
                darkturquoise:        '00ced1',
                darkviolet:           '9400d3',
                deeppink:             'ff1493',
                deepskyblue:          '00bfff',
                dimgray:              '696969',
                dimgrey:              '696969',
                dodgerblue:           '1e90ff',
                firebrick:            'b22222',
                floralwhite:          'fffaf0',
                forestgreen:          '228b22',
                fuchsia:              'f0f',
                gainsboro:            'dcdcdc',
                ghostwhite:           'f8f8ff',
                gold:                 'ffd700',
                goldenrod:            'daa520',
                gray:                 '808080',
                grey:                 '808080',
                green:                '008000',
                greenyellow:          'adff2f',
                honeydew:             'f0fff0',
                hotpink:              'ff69b4',
                indianred:            'cd5c5c',
                indigo:               '4b0082',
                ivory:                'fffff0',
                khaki:                'f0e68c',
                lavender:             'e6e6fa',
                lavenderblush:        'fff0f5',
                lawngreen:            '7cfc00',
                lemonchiffon:         'fffacd',
                lightblue:            'add8e6',
                lightcoral:           'f08080',
                lightcyan:            'e0ffff',
                lightgoldenrodyellow: 'fafad2',
                lightgray:            'd3d3d3',
                lightgrey:            'd3d3d3',
                lightgreen:           '90ee90',
                lightpink:            'ffb6c1',
                lightsalmon:          'ffa07a',
                lightseagreen:        '20b2aa',
                lightskyblue:         '87cefa',
                lightslategray:       '789',
                lightslategrey:       '789',
                lightsteelblue:       'b0c4de',
                lightyellow:          'ffffe0',
                lime:                 '0f0',
                limegreen:            '32cd32',
                linen:                'faf0e6',
                magenta:              'f0f',
                maroon:               '800000',
                mediumaquamarine:     '66cdaa',
                mediumblue:           '0000cd',
                mediumorchid:         'ba55d3',
                mediumpurple:         '9370d8',
                mediumseagreen:       '3cb371',
                mediumslateblue:      '7b68ee',
                mediumspringgreen:    '00fa9a',
                mediumturquoise:      '48d1cc',
                mediumvioletred:      'c71585',
                midnightblue:         '191970',
                mintcream:            'f5fffa',
                mistyrose:            'ffe4e1',
                moccasin:             'ffe4b5',
                navajowhite:          'ffdead',
                navy:                 '000080',
                oldlace:              'fdf5e6',
                olive:                '808000',
                olivedrab:            '6b8e23',
                orange:               'ffa500',
                orangered:            'ff4500',
                orchid:               'da70d6',
                palegoldenrod:        'eee8aa',
                palegreen:            '98fb98',
                paleturquoise:        'afeeee',
                palevioletred:        'd87093',
                papayawhip:           'ffefd5',
                peachpuff:            'ffdab9',
                peru:                 'cd853f',
                pink:                 'ffc0cb',
                plum:                 'dda0dd',
                powderblue:           'b0e0e6',
                purple:               '800080',
                rebeccapurple:        '639',
                red:                  'f00',
                rosybrown:            'bc8f8f',
                royalblue:            '4169e1',
                saddlebrown:          '8b4513',
                salmon:               'fa8072',
                sandybrown:           'f4a460',
                seagreen:             '2e8b57',
                seashell:             'fff5ee',
                sienna:               'a0522d',
                silver:               'c0c0c0',
                skyblue:              '87ceeb',
                slateblue:            '6a5acd',
                slategray:            '708090',
                slategrey:            '708090',
                snow:                 'fffafa',
                springgreen:          '00ff7f',
                steelblue:            '4682b4',
                tan:                  'd2b48c',
                teal:                 '008080',
                thistle:              'd8bfd8',
                tomato:               'ff6347',
                turquoise:            '40e0d0',
                violet:               'ee82ee',
                wheat:                'f5deb3',
                white:                'fff',
                whitesmoke:           'f5f5f5',
                yellow:               'ff0',
                yellowgreen:          '9acd32'
            };

        // change named color to corresponding hex value
        if (namedColors[color]) {
            color = namedColors[color];
        }

        // check val for hex color
        if (color.match(hexPattern) !== null) {

            // '#' entfernen wenn vorhanden
            color = color.replace('#', '');

            // color has either 3 or 6 characters
            if (color.length % 2 === 1) {

                // color has 3 char -> convert to 6 char
                // r = color.substr(0,1).repeat(2);
                // g = color.substr(1,1).repeat(2); // String.prototype.repeat() doesn't work in IE11
                // b = color.substr(2,1).repeat(2);
                r = String(color.substr(0, 1)) + color.substr(0, 1);
                g = String(color.substr(1, 1)) + color.substr(1, 1);
                b = String(color.substr(2, 1)) + color.substr(2, 1);

                result.rgb = {
                    r: parseInt(r, 16),
                    g: parseInt(g, 16),
                    b: parseInt(b, 16)
                };

                result.hex = `#${r}${g}${b}`;

            } else {

                // color has 6 char
                result.rgb = {
                    r: parseInt(color.substr(0, 2), 16),
                    g: parseInt(color.substr(2, 2), 16),
                    b: parseInt(color.substr(4, 2), 16)
                };

                result.hex = `#${color}`;

            }

            hsl = this.rgbToHsl(result.rgb.r, result.rgb.g, result.rgb.b);
            result.hsl = hsl;
            result.rgb.css = `rgb(${result.rgb.r},${result.rgb.g},${result.rgb.b})`;

        }
        // check val for rgb/rgba color
        else if (color.match(RGBAPattern)) {

            match = RGBAPattern.exec(color);
            result.rgb = {css: color, r: match[1], g: match[2], b: match[3]};
            result.hex = this.rgbToHex(match[1], match[2], match[3]);
            hsl = this.rgbToHsl(match[1], match[2], match[3]);
            result.hsl = hsl;

        }
        // check val for hsl/hsla color
        else if (color.match(HSLAPattern)) {

            match = HSLAPattern.exec(color);

            h = match[1] / 360;
            s = match[2].substr(0, match[2].length - 1) / 100;
            l = match[3].substr(0, match[3].length - 1) / 100;

            channels = this.hslToRgb(h, s, l);

            result.rgb = {
                css: `rgb(${channels[0]},${channels[1]},${channels[2]})`,
                r:   channels[0],
                g:   channels[1],
                b:   channels[2]
            };
            result.hex = this.rgbToHex(result.rgb.r, result.rgb.g, result.rgb.b);
            result.hsl = {css: `hsl(${match[1]},${match[2]},${match[3]})`, h: match[1], s: match[2], l: match[3]};

        }

        // or return #f5f5f5
        else {
            result.hex = '#f5f5f5';
            result.rgb = {css: 'rgb(245,245,245)', r: 245, g: 245, b: 245};
            result.hsl = {css: 'hsl(0,0%,96.08%)', h: 0, s: '0%', l: '96.08%'};
        }

        return result;

    },

    createPanelTemplate(dataAttr = true) {
        const panel = document.createElement('div');
        panel.className = 'jsPanel';
        if (dataAttr) {
            ['close', 'maximize', 'normalize', 'minimize', 'smallify', 'smallifyrev'].forEach((item) => {
                panel.setAttribute(`data-btn${item}`, 'enabled');
            });
        }
        panel.innerHTML = `<div class="jsPanel-hdr">
                                <div class="jsPanel-headerbar">
                                    <div class="jsPanel-headerlogo"></div>
                                    <div class="jsPanel-titlebar">
                                        <span class="jsPanel-title"></span>
                                    </div>
                                    <div class="jsPanel-controlbar">
                                        <div class="jsPanel-btn jsPanel-btn-smallify">${this.icons.smallify}</div>
                                        <div class="jsPanel-btn jsPanel-btn-smallifyrev">${this.icons.smallifyrev}</div>
                                        <div class="jsPanel-btn jsPanel-btn-minimize">${this.icons.minimize}</div>
                                        <div class="jsPanel-btn jsPanel-btn-normalize">${this.icons.normalize}</div>
                                        <div class="jsPanel-btn jsPanel-btn-maximize">${this.icons.maximize}</div>
                                        <div class="jsPanel-btn jsPanel-btn-close">${this.icons.close}</div>
                                    </div>
                                </div>
                                <div class="jsPanel-hdr-toolbar"></div>
                            </div>
                            <div class="jsPanel-content"></div>
                            <div class="jsPanel-minimized-box"></div>
                            <div class="jsPanel-ftr"></div>`;
        return panel;
    },

    createMinimizedTemplate() {
        const panel = document.createElement('div');
        panel.className = 'jsPanel-replacement';
        panel.innerHTML = `<div class="jsPanel-hdr">
                                <div class="jsPanel-headerbar">
                                    <div class="jsPanel-headerlogo"></div>
                                    <div class="jsPanel-titlebar">
                                        <span class="jsPanel-title"></span>
                                    </div>
                                    <div class="jsPanel-controlbar">
                                        <div class="jsPanel-btn jsPanel-btn-normalize">${this.icons.normalize}</div>
                                        <div class="jsPanel-btn jsPanel-btn-maximize">${this.icons.maximize}</div>
                                        <div class="jsPanel-btn jsPanel-btn-close">${this.icons.close}</div>
                                    </div>
                                </div>
                            </div>`;
        return panel;
    },

    createSnapArea(panel, pos, snapsens) {
        const el = document.createElement('div'),
            parent = panel.parentElement;
        el.className = `jsPanel-snap-area jsPanel-snap-area-${pos}`;
        if (pos === 'lt' || pos === 'rt' || pos === 'rb' || pos === 'lb') {
            el.style.width = snapsens+'px';
            el.style.height = snapsens+'px';
        } else if (pos === 'ct' || pos === 'cb') {
            el.style.height = snapsens+'px';
        } else if (pos === 'lc' || pos === 'rc') {
            el.style.width = snapsens+'px';
        }
        if (parent !== document.body) {
            el.style.position = 'absolute';
        }
        if (!document.querySelector(`.jsPanel-snap-area.jsPanel-snap-area-${pos}`)) {
            panel.parentElement.appendChild(el);
        }
    },

    darken(val, amount) {
        // amount is value between 0 and 1
        const hsl = this.color(val).hsl,
            l = parseFloat(hsl.l),
            lnew = l - (l * amount) + '%';
        return `hsl(${hsl.h},${hsl.s},${lnew})`;
    },

    dragit(elmt, options = {}) {
        let dragstarted,
            opts = Object.assign({}, this.defaults.dragit, options),
            dragElmt,
            containment;
        const jspaneldragstart = new CustomEvent('jspaneldragstart', {detail: elmt.id}),
            jspaneldrag      = new CustomEvent('jspaneldrag',      {detail: elmt.id}),
            jspaneldragstop  = new CustomEvent('jspaneldragstop',  {detail: elmt.id});

        // normalize grid config
        if (opts.grid && Array.isArray(opts.grid)) {
            if (opts.grid.length === 1) {
                opts.grid[1] = opts.grid[0];
            }
        }

        // normalize containment config
        containment = this.pOcontainment(opts.containment);

        // attach handler to each drag handle
        elmt.querySelectorAll(opts.handles).forEach((handle) => {

            handle.style.touchAction = 'none';
            handle.style.cursor = opts.cursor;

            jsPanel.pointerdown.forEach((item) => {
                handle.addEventListener(item, (e) => {
                    // prevent body scroll on drag init
                    e.preventDefault();

                    // disable draging for all mouse buttons but left
                    if (e.button && e.button > 0) { return false; }

                    // footer elmts with the class "jsPanel-ftr-btn" don't drag a panel
                    // do not compare e.target with e.currentTarget because there might be footer elmts supposed to drag the panel
                    if (e.target.closest('.jsPanel-ftr-btn')) {return;}

                    elmt.controlbar.style.pointerEvents = 'none';
                    elmt.content.style.pointerEvents = 'none';  // without this code handler might not be unbound when content has iframe or object tag

                    const startStyles  = window.getComputedStyle(elmt),
                        startLeft    = parseFloat(startStyles.left),
                        startTop     = parseFloat(startStyles.top),
                        psx          = e.touches ? e.touches[0].clientX : e.clientX,    // pointer x on mousedown (don't use pageX, doesn't work on FF for Android)
                        psy          = e.touches ? e.touches[0].clientY : e.clientY,    // pointer y on mousedown (don't use pageY, doesn't work on FF for Android)
                        parent       = elmt.parentElement,
                        parentRect   = parent.getBoundingClientRect(),
                        parentStyles = window.getComputedStyle(parent);
                    let startLeftCorrection = 0;

                    // function actually draging the elmt
                    dragElmt = (e) => {
                        e.preventDefault();

                        if (!dragstarted) {
                            document.dispatchEvent(jspaneldragstart);
                            elmt.style.opacity = opts.opacity;
                            // if configured restore panel size to values before snap and reposition reasonable before drag actually starts
                            if (elmt.snapped && opts.snap.resizeToPreSnap && elmt.currentData.beforeSnap) {
                                elmt.resize(elmt.currentData.beforeSnap.width + ' ' + elmt.currentData.beforeSnap.height);
                                let intermediateStyles = elmt.getBoundingClientRect(),
                                    delta = psx - (intermediateStyles.left + intermediateStyles.width),
                                    wHalf = intermediateStyles.width/2;
                                if (delta > -wHalf) {
                                    startLeftCorrection = delta + wHalf;
                                }
                            }
                            // dragstart callback
                            if (opts.start) {
                                jsPanel.processCallbacks(elmt, opts.start, false, {left: startLeft, top: startTop});
                            }
                            jsPanel.front(elmt);
                            elmt.snapped = false;
                        }
                        dragstarted = 1;

                        if (opts.disableOnMaximized && elmt.status === 'maximized') {
                            return false;
                        }

                        let elmtL, elmtL2, elmtT, elmtT2, elmtR, elmtR2, elmtB, elmtB2, right, bottom;
                        const pmx = e.touches ? e.touches[0].clientX : e.clientX,    // current pointer x while pointer moves (don't use pageX, doesn't work on FF for Android)
                            pmy   = e.touches ? e.touches[0].clientY : e.clientY,    // current pointer y while pointer moves (don't use pageY, doesn't work on FF for Android)
                            dragStyles = window.getComputedStyle(elmt);             // get current styles while draging

                        // EDGE reports "auto" instead of pixel value using getComputedStyle(), so some values need to be calculated different
                        if (parent === document.body) {
                            let elmtRect = elmt.getBoundingClientRect();
                            right = window.innerWidth - parseInt(parentStyles.borderLeftWidth, 10) - parseInt(parentStyles.borderRightWidth, 10) - (elmtRect.left + elmtRect.width);
                            bottom = window.innerHeight - parseInt(parentStyles.borderTopWidth, 10) - parseInt(parentStyles.borderBottomWidth, 10) - (elmtRect.top + elmtRect.height);
                        } else {
                            right = parseInt(parentStyles.width, 10) - parseInt(parentStyles.borderLeftWidth, 10) - parseInt(parentStyles.borderRightWidth, 10) - (parseInt(dragStyles.left, 10) + parseInt(dragStyles.width, 10));
                            bottom = parseInt(parentStyles.height, 10) - parseInt(parentStyles.borderTopWidth, 10) - parseInt(parentStyles.borderBottomWidth, 10) - (parseInt(dragStyles.top, 10) + parseInt(dragStyles.height, 10));
                        }
                        // xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

                        elmtL = parseFloat(dragStyles.left);
                        elmtT = parseFloat(dragStyles.top);
                        elmtR = right; // instead of parseFloat(dragStyles.right);
                        elmtB = bottom; // instead of parseFloat(dragStyles.bottom);

                        if (opts.snap) {
                            if (opts.snap.trigger === 'panel') {
                                elmtL2 = elmtL ** 2;
                            } else if (opts.snap.trigger === 'pointer') {
                                elmtL = pmx;
                                elmtL2 = pmx ** 2;
                                elmtT = pmy;
                                elmtR = window.innerWidth - pmx;
                                elmtB = window.innerHeight - pmy;
                            }
                            elmtT2 = elmtT ** 2;
                            elmtR2 = elmtR ** 2;
                            elmtB2 = elmtB ** 2;
                        }

                        const lefttopVectorDrag   = Math.sqrt(elmtL2 + elmtT2),
                            leftbottomVectorDrag  = Math.sqrt(elmtL2 + elmtB2),
                            righttopVectorDrag    = Math.sqrt(elmtR2 + elmtT2),
                            rightbottomVectorDrag = Math.sqrt(elmtR2 + elmtB2),
                            horizontalDeltaDrag   = Math.abs(elmtL - elmtR)/2,
                            verticalDeltaDrag     = Math.abs(elmtT - elmtB)/2,
                            leftVectorDrag        = Math.sqrt(elmtL2 + verticalDeltaDrag ** 2),
                            topVectorDrag         = Math.sqrt(elmtT2 + horizontalDeltaDrag ** 2),
                            rightVectorDrag       = Math.sqrt(elmtR2 + verticalDeltaDrag ** 2),
                            bottomVectorDrag      = Math.sqrt(elmtB2 + horizontalDeltaDrag ** 2);

                        // prevent selctions while draging
                        window.getSelection().removeAllRanges();

                        // trigger drag permanently while draging
                        document.dispatchEvent(jspaneldrag);

                        // move elmt
                        if (!opts.axis || opts.axis === 'x') {
                            elmt.style.left = startLeft + (pmx - psx) + startLeftCorrection + 'px';    // set new css left of elmt depending on opts.axis
                        }
                        if (!opts.axis || opts.axis === 'y') {
                            elmt.style.top = startTop + (pmy - psy) + 'px';      // set new css top of elmt depending on opts.axis
                        }

                        // apply grid option
                        if (opts.grid) {
                            const cx   = parseFloat(dragStyles.left),
                                cy   = parseFloat(dragStyles.top),
                                modX = cx % opts.grid[0],
                                modY = cy % opts.grid[1];
                            if (modX < opts.grid[0] / 2) {
                                elmt.style.left = (cx - modX) + 'px';
                            } else {
                                elmt.style.left = (cx + (opts.grid[0] - modX)) + 'px';
                            }
                            if (modY < opts.grid[1] / 2) {
                                elmt.style.top = (cy - modY) + 'px';
                            } else {
                                elmt.style.top = (cy + (opts.grid[1] - modY)) + 'px';
                            }
                        }

                        // apply containment option
                        if (opts.containment || opts.containment === 0) {
                            let maxLeft, maxTop;

                            // calc maxLeft and maxTop (minLeft and MinTop is equal to containment setting)
                            if (elmt.options.container === document.body) {
                                maxLeft = window.innerWidth - parseFloat(dragStyles.width) - containment[1];
                                maxTop = window.innerHeight - parseFloat(dragStyles.height) - containment[2];
                            } else {
                                const xCorr = parseFloat(parentStyles.borderLeftWidth) + parseFloat(parentStyles.borderRightWidth);
                                const yCorr = parseFloat(parentStyles.borderTopWidth) + parseFloat(parentStyles.borderBottomWidth);
                                maxLeft = parentRect.width - parseFloat(dragStyles.width) - containment[1] - xCorr;
                                maxTop = parentRect.height - parseFloat(dragStyles.height) - containment[2] - yCorr;
                            }

                            if (parseFloat(elmt.style.left) <= containment[3]) {
                                elmt.style.left = containment[3] + 'px';
                            }
                            if (parseFloat(elmt.style.top) <= containment[0]) {
                                elmt.style.top = containment[0] + 'px';
                            }
                            if (parseFloat(elmt.style.left) >= maxLeft) {
                                elmt.style.left = maxLeft + 'px';
                            }
                            if (parseFloat(elmt.style.top) >= maxTop) {
                                elmt.style.top = maxTop + 'px';
                            }
                        }

                        // callback while dragging
                        if (opts.drag) {
                            jsPanel.processCallbacks(elmt, opts.drag, false, {left: elmtL, top: elmtT, right: elmtR, bottom: elmtB});
                        }

                        // apply snap options
                        if (opts.snap) {
                            const snapSens = opts.snap.sensitivity,
                                topSensAreaLength = parent === document.body ? window.innerWidth/8 : parentRect.width/8,
                                sideSensAreaLength = parent === document.body ? window.innerHeight/8 : parentRect.height/8;
                            elmt.snappableTo = false;
                            jsPanel.removeSnapAreas(elmt);

                            if (lefttopVectorDrag < snapSens) {
                                elmt.snappableTo = 'left-top';
                                if (opts.snap.snapLeftTop !== false) {
                                    jsPanel.createSnapArea(elmt, 'lt', snapSens);
                                }
                            } else if (leftbottomVectorDrag < snapSens) {
                                elmt.snappableTo = 'left-bottom';
                                if (opts.snap.snapLeftBottom !== false) {
                                    jsPanel.createSnapArea(elmt, 'lb', snapSens);
                                }
                            } else if (righttopVectorDrag < snapSens) {
                                elmt.snappableTo = 'right-top';
                                if (opts.snap.snapRightTop !== false) {
                                    jsPanel.createSnapArea(elmt, 'rt', snapSens);
                                }
                            } else if (rightbottomVectorDrag < snapSens) {
                                elmt.snappableTo = 'right-bottom';
                                if (opts.snap.snapRightBottom !== false) {
                                    jsPanel.createSnapArea(elmt, 'rb', snapSens);
                                }
                            } else if (elmtT < snapSens && topVectorDrag < topSensAreaLength) {
                                elmt.snappableTo = 'center-top';
                                if (opts.snap.snapCenterTop !== false) {
                                    jsPanel.createSnapArea(elmt, 'ct', snapSens);
                                }
                            } else if (elmtL < snapSens && leftVectorDrag < sideSensAreaLength) {
                                elmt.snappableTo = 'left-center';
                                if (opts.snap.snapLeftCenter !== false) {
                                    jsPanel.createSnapArea(elmt, 'lc', snapSens);
                                }
                            } else if (elmtR < snapSens && rightVectorDrag < sideSensAreaLength) {
                                elmt.snappableTo = 'right-center';
                                if (opts.snap.snapRightCenter !== false) {
                                    jsPanel.createSnapArea(elmt, 'rc', snapSens);
                                }
                            } else if (elmtB < snapSens && bottomVectorDrag < topSensAreaLength) {
                                elmt.snappableTo = 'center-bottom';
                                if (opts.snap.snapCenterBottom !== false) {
                                    jsPanel.createSnapArea(elmt, 'cb', snapSens);
                                }
                            }
                        }

                    };

                    jsPanel.pointermove.forEach(function (item) {
                        document.addEventListener(item, dragElmt);
                    });

                });
            });

            jsPanel.pointerup.forEach(function (item) {
                document.addEventListener(item, () => {

                    jsPanel.pointermove.forEach(function (item) {
                        document.removeEventListener(item, dragElmt);
                    });

                    document.body.style.overflow = 'inherit';
                    jsPanel.removeSnapAreas(elmt);

                    if (dragstarted) {
                        document.dispatchEvent(jspaneldragstop);
                        elmt.style.opacity = 1;
                        dragstarted = undefined;
                        elmt.saveCurrentPosition();

                        if (opts.snap) {
                            if (elmt.snappableTo === 'left-top') {
                                jsPanel.snapPanel(elmt, opts.snap.snapLeftTop);
                            } else if (elmt.snappableTo === 'center-top') {
                                jsPanel.snapPanel(elmt, opts.snap.snapCenterTop);
                            } else if (elmt.snappableTo === 'right-top') {
                                jsPanel.snapPanel(elmt, opts.snap.snapRightTop);
                            } else if (elmt.snappableTo === 'right-center') {
                                jsPanel.snapPanel(elmt, opts.snap.snapRightCenter);
                            } else if (elmt.snappableTo === 'right-bottom') {
                                jsPanel.snapPanel(elmt, opts.snap.snapRightBottom);
                            } else if (elmt.snappableTo === 'center-bottom') {
                                jsPanel.snapPanel(elmt, opts.snap.snapCenterBottom);
                            } else if (elmt.snappableTo === 'left-bottom') {
                                jsPanel.snapPanel(elmt, opts.snap.snapLeftBottom);
                            } else if (elmt.snappableTo === 'left-center') {
                                jsPanel.snapPanel(elmt, opts.snap.snapLeftCenter);
                            }

                            if (opts.snap.callback && elmt.snappableTo && typeof opts.snap.callback === 'function') {
                                opts.snap.callback.call(elmt, elmt);
                            }

                            if (elmt.snappableTo && opts.snap.repositionOnSnap) {
                                elmt.repositionOnSnap(elmt.snappableTo);
                            }
                        }

                        if (opts.stop) {
                            jsPanel.processCallbacks(elmt, opts.stop, false, {left: parseFloat(elmt.style.left), top: parseFloat(elmt.style.top)});
                        }
                    }

                    elmt.controlbar.style.pointerEvents = 'inherit';
                    elmt.content.style.pointerEvents = 'inherit';

                });
            });

            // dragit is initialized - now disable if set
            if (opts.disable) {handle.style.pointerEvents = 'none';}

        });

        return elmt;
    },

    emptyNode(node) {
        while (node.firstChild) {node.removeChild(node.firstChild);}
        return node;
    },

    extend(obj) {
        // obj needs to be a plain object (to extend the individual panel, not the global object)
        if (Object.prototype.toString.call(obj) === '[object Object]') {
            for (let ext in obj) {
                if (obj.hasOwnProperty(ext)) {
                    this.extensions[ext] = obj[ext];
                }
            }
        }
    },

    fetch(obj) {
        let conf = obj.options.contentFetch;
        let confDefaults = {
            bodyMethod: 'text',
            evalscripttags: true,
            autoresize: true,
            autoreposition: true,
            done: function (obj, response) {
                obj.content.innerHTML = response;
            }
        };

        if (typeof conf === 'string') {
            conf = Object.assign({resource: obj.options.contentFetch}, confDefaults);
        } else {
            conf = Object.assign(confDefaults, conf);
        }
        let fetchInit = conf.fetchInit || {};

        if (conf.beforeSend) {
            conf.beforeSend.call(obj, obj);
        }

        fetch(conf.resource, fetchInit)
            .then(function(response) {

                if(response.ok) {
                    return response[conf.bodyMethod]();
                }
                throw new Error('Network response was not ok.');

            }).then(function(response){

                conf.done.call(obj, obj, response);

                // extract and eval content of script tags if "evalscripttags"
                if (conf.evalscripttags) {
                    // get all script tags within responseText
                    const scripttags = response.match(/<script\b[^>]*>([\s\S]*?)<\/script>/gi);
                    if (scripttags) {
                        scripttags.forEach((tag) => {
                            // remove tags from string and trim it
                            let js = tag.replace(/<script\b[^>]*>/i, '').replace(/<\/script>/i, '').trim();
                            // execute javascript
                            eval(js);
                        });
                    }
                }

                // resize and reposition panel if either width or height is set to 'auto'
                const oContentSize = obj.options.contentSize;
                if (conf.autoresize || conf.autoreposition) {
                    if (typeof oContentSize === 'string' && oContentSize.match(/auto/i)) {
                        const parts = oContentSize.split(' ');
                        const sizes = Object.assign({}, {width: parts[0], height: parts[1]});
                        if (conf.autoresize) {
                            obj.resize(sizes);
                        }
                        if (!obj.classList.contains('jsPanel-contextmenu')) {
                            if (conf.autoreposition) {
                                obj.reposition();
                            }
                        }
                    } else if (typeof oContentSize === 'object' && (oContentSize.width === 'auto' || oContentSize.height === 'auto')) {
                        const sizes = Object.assign({}, oContentSize);
                        if (conf.autoresize) {
                            obj.resize(sizes);
                        }
                        if (!obj.classList.contains('jsPanel-contextmenu')) {
                            if (conf.autoreposition) {
                                obj.reposition();
                            }
                        }
                    }
                }

            }).catch(function(error) {
                console.error('There has been a problem with your fetch operation: ' + error.message);
            });

    },

    front(obj) {
        if (obj.status === 'minimized') {
            if (obj.statusBefore === 'maximized') {
                obj.maximize();
            } else {
                obj.normalize();
            }
        } else {
            const newArr = Array.prototype.slice.call(document.querySelectorAll('.jsPanel-standard')).map((panel) => {
                return panel.style.zIndex;
            });
            if (Math.max(...newArr) > obj.style.zIndex) {
                obj.style.zIndex = jsPanel.zi.next();
            }
            this.resetZi();
        }

        // iframe extra on test: panel with an iframe get a content overlay if panel is not the topmost panel -> content section click fronts the panel
        this.getPanels().forEach(function(item, index) {
            let overlay = item.content.querySelector('.jsPanel-iframe-overlay');
            if (index > 0) {
                if (item.content.querySelector('iframe') && !overlay) {
                    let overlay = document.createElement('div');
                    overlay.className = 'jsPanel-iframe-overlay';
                    item.content.appendChild(overlay);
                }
            } else {
                if (overlay) {
                    item.content.removeChild(overlay);
                }
            }
        });
    },

    getPanels(condition = function() {return this.classList.contains('jsPanel-standard');}) {
        return Array.prototype.slice.call(document.querySelectorAll('.jsPanel'))
            .filter((value) => {
                return condition.call(value, value);
            }).sort((a, b) => {
                return b.style.zIndex - a.style.zIndex;
            });
    },

    hslToRgb(h, s, l){
        // h, s and l must be values between 0 and 1
        let r, g, b;
        if (s === 0) {
            r = g = b = l; // achromatic
        } else {
            let hue2rgb = (p, q, t) => {
                if (t < 0) {
                    t += 1;
                }
                if (t > 1) {
                    t -= 1;
                }
                if (t < 1 / 6) {
                    return p + (q - p) * 6 * t;
                }
                if (t < 1 / 2) {
                    return q;
                }
                if (t < 2 / 3) {
                    return p + (q - p) * (2 / 3 - t) * 6;
                }
                return p;
            };
            let q = l < 0.5 ? l * (1 + s) : l + s - l * s,
                p = 2 * l - q;
            r = hue2rgb(p, q, h + 1 / 3);
            g = hue2rgb(p, q, h);
            b = hue2rgb(p, q, h - 1 / 3);
        }
        return [Math.round(r * 255), Math.round(g * 255), Math.round(b * 255)];
    },

    lighten(val, amount) {
        // amount is value between 0 and 1
        const hsl = this.color(val).hsl,
            l = parseFloat(hsl.l),
            lnew = (l + ((100 - l) * amount)) + '%';
        return `hsl(${hsl.h},${hsl.s},${lnew})`;
    },

    perceivedBrightness(val) {
        const rgb = this.color(val).rgb;
        // return value is in the range 0 - 1 and input rgb values must also be in the range 0 - 1
        // algorithm from: https://en.wikipedia.org/wiki/Rec._2020
        return ( (rgb.r / 255 * 0.2627) + (rgb.g / 255 * 0.6780) + (rgb.b / 255 * 0.0593) );
    },

    pOcontainer(container, cb) {
        if (container) {
            let box;
            if (typeof container === 'string') {
                box = document.querySelector(container);
            } else if (container.nodeType === 1) {
                box = container;
            } else if (container.length) {
                box = container[0];
            }
            if (box && box.nodeType === 1) {
                return box;
            }
        }

        const error = new window.jsPanelError('NO NEW PANEL CREATED!\nThe container to append the panel to does not exist or a container was not specified!');
        try {
            throw error;
        } catch (e) {
            if (cb) {cb.call(e, e);}
        }
        return error;
    },

    // normalizes values for option.maximizedMargin and containment of dragit/resizeit
    pOcontainment(arg) {
        if (typeof arg === 'number') {
            // arg: 20 => arg: [20, 20, 20, 20]
            return [arg,arg,arg,arg];
        } else if (Array.isArray(arg)) {
            if (arg.length === 1) {
                // arg: [20] => arg: [20, 20, 20, 20]
                return [arg[0],arg[0],arg[0],arg[0]];
            } else if (arg.length === 2) {
                // arg: [20, 40] => arg: [20, 40, 20, 40]
                return arg.concat(arg);
            } else if (arg.length === 3) {
                arg[3] = arg[1];
            }
        }
        return arg; // assumed to be array with 4 values
    },

    pOsize(panel, size) {
        let values = size || this.defaults.contentSize,
            parent = panel.parentElement;
        if (typeof values === 'string') {
            let nums = values.trim().split(' ');
            values = {};
            values.width = nums[0];
            nums.length === 2 ? values.height = nums[1] : values.height = nums[0];
        } else {
            if (values.width && !values.height) {
                values.height = values.width;
            } else if (values.height && !values.width) {
                values.width = values.height;
            }
        }

        if (String(values.width).match(/^[0-9.]+$/gi)) {
            // if number only
            values.width += 'px';
        } else if (typeof values.width === 'string' && values.width.endsWith('%')) {
            if (parent === document.body) {
                values.width = window.innerWidth * (parseFloat(values.width)/100) + 'px';
            } else {
                const prtStyles = window.getComputedStyle(parent),
                    border = parseFloat(prtStyles.borderLeftWidth) + parseFloat(prtStyles.borderRightWidth);
                values.width = (parseFloat(prtStyles.width) - border) * (parseFloat(values.width)/100) + 'px';
            }
        } else if (typeof values.width === 'function') {
            values.width = values.width.call(panel, panel);
            if (typeof values.width === 'number') {
                values.width += 'px';
            } else if (typeof values.width === 'string' && values.width.match(/^[0-9.]+$/gi)) {
                values.width += 'px';
            }
        }

        if (String(values.height).match(/^[0-9.]+$/gi)) {
            // if number only
            values.height += 'px';
        } else if (typeof values.height === 'string' && values.height.endsWith('%')) {
            if (parent === document.body) {
                values.height = window.innerHeight * (parseFloat(values.height)/100) + 'px';
            } else {
                const prtStyles = window.getComputedStyle(parent),
                    border = parseFloat(prtStyles.borderTopWidth) + parseFloat(prtStyles.borderBottomWidth);
                values.height = (parseFloat(prtStyles.height) - border) * (parseFloat(values.height)/100) + 'px';
            }
        } else if (typeof values.height === 'function') {
            values.height = values.height.call(panel, panel);
            if (typeof values.height === 'number') {
                values.height += 'px';
            } else if (typeof values.height === 'string' && values.height.match(/^[0-9.]+$/gi)) {
                values.height += 'px';
            }
        }

        return values; // return value must be object {width: xxx, height: xxx}
    },

    pOposition(positionString) {
        const posValue   = positionString.match(/\b[a-z]{4,6}-{1}[a-z]{3,6}\b/i),
            autoposValue = positionString.match(/down|up|right([^-]|$)|left([^-]|$)/i),
            offsetValue  = positionString.match(/[+-]?\d?\.?\d+([a-z%]{2,4}\b|%?)/gi);
        let settings;

        if (posValue) {
            settings = {my: posValue[0].toLowerCase(), at: posValue[0].toLowerCase()};
        } else {
            settings = {my: 'center', at: 'center'};
        }

        if (autoposValue) {
            settings.autoposition = autoposValue[0].toLowerCase();
        }

        if (offsetValue) {
            // convert strings with only numbers to a number value
            offsetValue.forEach((item, index) => {
                if (item.match(/^[+-]?[0-9]*$/)) {
                    offsetValue[index] += 'px';
                }
                offsetValue[index] = offsetValue[index].toLowerCase();
            });
            // only one passed offset is used for both offsetX and offsetY
            if (offsetValue.length === 1) {
                settings.offsetX = offsetValue[0];
                settings.offsetY = offsetValue[0];
            } else {
                settings.offsetX = offsetValue[0];
                settings.offsetY = offsetValue[1];
            }
        }

        return settings;
    },

    position(elmt, position) {
        let elmtToPosition,
            posSettings,
            elmtToPositionAgainst,
            calculatedPosition = {left:0, top:0},
            myXcorrection = 0, myYcorrection = 0,
            atXcorrection = 0, atYcorrection = 0;

        const defaults = {my:'center', at:'center', of:'window', offsetX:'0px', offsetY:'0px'},
            windowRect = {
                width: document.documentElement.clientWidth,
                height: window.innerHeight
            },
            scrollX = pageXOffset,
            scrollY = pageYOffset;

        if (typeof elmt === 'string') {
            // arg elmt is assumed to be a selector string
            elmtToPosition = document.querySelector(elmt);
        } else {
            // otherwise arg elmt is assumed to be a node object
            elmtToPosition = elmt;
        }

        // do not position elmt when parameter position is set to boolean false
        if (!position) {
            elmtToPosition.style.opacity = 1;
            return elmtToPosition;
        }

        const elmtToPositionRect = elmtToPosition.getBoundingClientRect();
        // contains read-only left, top, right, bottom, x, y, width, height describing the !! border-box !! in pixels
        // Properties other than width and height are relative to the top-left of the viewport!!

        // translate shorthand string to object - "top-left 50 50 down"
        if (typeof position === 'string') {
            posSettings = Object.assign({}, defaults, jsPanel.pOposition(position));
        } else {
            posSettings = Object.assign({}, defaults, position);
        }

        const parentContainer        = elmtToPosition.parentElement;
        const parentContainerStyles  = window.getComputedStyle(parentContainer);
        const parentContainerRect    = parentContainer.getBoundingClientRect();
        const parentContainerTagName = parentContainer.tagName.toLowerCase();

        if (posSettings.of && posSettings.of !== 'window') {
            if (typeof posSettings.of === 'function') {
                posSettings.of = posSettings.of();
            }
            if (typeof posSettings.of === 'string') {
                // posSettings.of is assumed to be a selector string
                elmtToPositionAgainst = document.querySelector(posSettings.of);
            } else {
                // otherwise posSettings.of is assumed to be a node object
                elmtToPositionAgainst = posSettings.of;
            }
        }

        // process my/at functions if applicable
        if (typeof posSettings.my === 'function') {
            posSettings.my = posSettings.my(); // function must return a proper string value
        }
        if (typeof posSettings.at === 'function') {
            posSettings.at = posSettings.at(); // function must return a proper string value
        }

        // calc left corrections due to panel size, should be the same for all scenarios
        if (posSettings.my.match(/^center-top$|^center$|^center-bottom$/i)) {
            myXcorrection = elmtToPositionRect.width/2;
        } else if (posSettings.my.match(/right/i)) {
            myXcorrection = elmtToPositionRect.width;
        }
        // calc top corrections due to panel size
        if (posSettings.my.match(/^left-center$|^center$|^right-center$/i)) {
            myYcorrection = elmtToPositionRect.height/2;
        } else if (posSettings.my.match(/bottom/i)) {
            myYcorrection = elmtToPositionRect.height;
        }

        // SCENARIO 1 - panel appended to body and positioned relative to window -> make fixed
        if (parentContainerTagName === 'body' && posSettings.of === 'window') {
            // calc left corrections due to window size
            if (posSettings.at.match(/^center-top$|^center$|^center-bottom$/i)) {
                atXcorrection = windowRect.width/2;
            } else if (posSettings.at.match(/right/i)) {
                atXcorrection = windowRect.width;
            }
            // calc top corrections due to window size
            if (posSettings.at.match(/^left-center$|^center$|^right-center$/i)) {
                atYcorrection = windowRect.height/2;
            } else if (posSettings.at.match(/bottom/i)) {
                atYcorrection = windowRect.height;
            }

            calculatedPosition.left = (atXcorrection - myXcorrection) - parseFloat(parentContainerStyles.borderLeftWidth);
            calculatedPosition.top  = (atYcorrection - myYcorrection) - parseFloat(parentContainerStyles.borderTopWidth);

            // panel appended to body and positioned relative to window is always fixed
            elmtToPosition.style.position = 'fixed';
        }

        // SCENARIO 2 - panel appended to body and positioned relative to another element in document
        else if (parentContainerTagName === 'body' && posSettings.of !== 'window') {
            const elmtToPositionAgainstRect = elmtToPositionAgainst.getBoundingClientRect();

            // calc left corrections due to position and size of elmtToPositionAgainst
            if (posSettings.at.match(/^center-top$|^center$|^center-bottom$/i)) {
                atXcorrection = elmtToPositionAgainstRect.width/2 + elmtToPositionAgainstRect.left + scrollX;
            } else if (posSettings.at.match(/right/i)) {
                atXcorrection = elmtToPositionAgainstRect.width + elmtToPositionAgainstRect.left + scrollX;
            } else {
                atXcorrection = elmtToPositionAgainstRect.left + scrollX;
            }
            // calc top corrections due to position and size of elmtToPositionAgainst
            if (posSettings.at.match(/^left-center$|^center$|^right-center$/i)) {
                atYcorrection = elmtToPositionAgainstRect.height/2 + elmtToPositionAgainstRect.top + scrollY;
            } else if (posSettings.at.match(/bottom/i)) {
                atYcorrection = elmtToPositionAgainstRect.height + elmtToPositionAgainstRect.top + scrollY;
            } else {
                atYcorrection = elmtToPositionAgainstRect.top + scrollY;
            }

            calculatedPosition.left = (atXcorrection - myXcorrection) - parseFloat(parentContainerStyles.borderLeftWidth);
            calculatedPosition.top  = (atYcorrection - myYcorrection) - parseFloat(parentContainerStyles.borderTopWidth);
        }

        // SCENARIO 3 - // panel appended to other element than body and positioned relative to its container
        else if (parentContainerTagName !== 'body' && (posSettings.of === 'window' || !posSettings.of)) {
            // calc corrections to position panel relative to parentContainer content-box, not border-box
            const pContainerLRBorderWidth = parseFloat(parentContainerStyles.borderLeftWidth) + parseFloat(parentContainerStyles.borderRightWidth),
                pContainerTBBorderWidth = parseFloat(parentContainerStyles.borderTopWidth) + parseFloat(parentContainerStyles.borderBottomWidth);

            // calc left corrections due to parent container width
            if (posSettings.at.match(/^center-top$|^center$|^center-bottom$/i)) {
                atXcorrection = parentContainerRect.width/2 - pContainerLRBorderWidth/2;
            } else if (posSettings.at.match(/right/i)) {
                atXcorrection = parentContainerRect.width - pContainerLRBorderWidth;
            }
            // calc top corrections due to parent container height
            if (posSettings.at.match(/^left-center$|^center$|^right-center$/i)) {
                atYcorrection = parentContainerRect.height/2 - pContainerTBBorderWidth/2;
            } else if (posSettings.at.match(/bottom/i)) {
                atYcorrection = parentContainerRect.height - pContainerTBBorderWidth;
            }

            calculatedPosition.left = (atXcorrection - myXcorrection);
            calculatedPosition.top  = (atYcorrection - myYcorrection);
        }

        // SCENARIO 4 - panel appended to other element than body and positioned relative to an element within its container
        else if (parentContainerTagName !== 'body' && parentContainer.contains(elmtToPositionAgainst)) {
            const elmtToPositionAgainstRect = elmtToPositionAgainst.getBoundingClientRect();

            // calc left corrections due to position and size of elmtToPositionAgainst
            if (posSettings.at.match(/^center-top$|^center$|^center-bottom$/i)) {
                atXcorrection = (elmtToPositionAgainstRect.left - parentContainerRect.left) + elmtToPositionAgainstRect.width/2;
            } else if (posSettings.at.match(/right/i)) {
                atXcorrection = (elmtToPositionAgainstRect.left - parentContainerRect.left) + elmtToPositionAgainstRect.width;
            } else {
                atXcorrection = (elmtToPositionAgainstRect.left - parentContainerRect.left);
            }
            // calc top corrections due to position and size of elmtToPositionAgainst
            if (posSettings.at.match(/^left-center$|^center$|^right-center$/i)) {
                atYcorrection = (elmtToPositionAgainstRect.top - parentContainerRect.top) + elmtToPositionAgainstRect.height/2;
            } else if (posSettings.at.match(/bottom/i)) {
                atYcorrection = (elmtToPositionAgainstRect.top - parentContainerRect.top) + elmtToPositionAgainstRect.height;
            } else {
                atYcorrection = (elmtToPositionAgainstRect.top - parentContainerRect.top);
            }

            calculatedPosition.left = (atXcorrection - myXcorrection) - parseFloat(parentContainerStyles.borderLeftWidth);
            calculatedPosition.top  = (atYcorrection - myYcorrection) - parseFloat(parentContainerStyles.borderTopWidth);
        }

        // autoposition panels only if ...
        if (posSettings.autoposition && posSettings.my === posSettings.at && ['left-top','center-top','right-top','left-bottom','center-bottom','right-bottom'].indexOf(posSettings.my) >= 0) {

            if (typeof posSettings.autoposition === 'function') {
                posSettings.autoposition = posSettings.autoposition(); // function must return a proper string value
            }

            // add class with position and autoposition direction
            const newClass = `${posSettings.my}-${posSettings.autoposition.toLowerCase()}`;
            elmtToPosition.classList.add(newClass);

            // get all panels with same class
            const newClassAll = Array.prototype.slice.call(document.querySelectorAll(`.${newClass}`)),
                ownIndex = newClassAll.indexOf(elmtToPosition);

            // if more than 1 position new panel
            if (newClassAll.length > 1) {
                if (posSettings.autoposition === 'down') {
                    // collect heights of all elmts to calc new top position
                    newClassAll.forEach((item, index) => {
                        if (index > 0 && index <= ownIndex) {
                            calculatedPosition.top += newClassAll[--index].getBoundingClientRect().height + jsPanel.autopositionSpacing;
                        }
                    });
                } else if (posSettings.autoposition === 'up') {
                    newClassAll.forEach((item, index) => {
                        if (index > 0 && index <= ownIndex) {
                            calculatedPosition.top -= newClassAll[--index].getBoundingClientRect().height + jsPanel.autopositionSpacing;
                        }
                    });
                } else if (posSettings.autoposition === 'right') {
                    // collect widths of all elmts to calc new left position
                    newClassAll.forEach((item, index) => {
                        if (index > 0 && index <= ownIndex) {
                            calculatedPosition.left += newClassAll[--index].getBoundingClientRect().width + jsPanel.autopositionSpacing;
                        }
                    });
                } else if (posSettings.autoposition === 'left') {
                    newClassAll.forEach((item, index) => {
                        if (index > 0 && index <= ownIndex) {
                            calculatedPosition.left -= newClassAll[--index].getBoundingClientRect().width + jsPanel.autopositionSpacing;
                        }
                    });
                }
            }
        }

        // convert positioning numbers to pixel values and position panel
        calculatedPosition.left += 'px';
        calculatedPosition.top += 'px';
        elmtToPosition.style.left = calculatedPosition.left;
        elmtToPosition.style.top = calculatedPosition.top;

        // apply offsets
        if (posSettings.offsetX) {

            if (typeof posSettings.offsetX === 'function') {
                posSettings.offsetX = posSettings.offsetX(); // function must return a proper number or string
            }

            typeof posSettings.offsetX === 'number'
                ? elmtToPosition.style.left = `calc(${calculatedPosition.left} + ${posSettings.offsetX}px)`
                : elmtToPosition.style.left = `calc(${calculatedPosition.left} + ${posSettings.offsetX})`;
            calculatedPosition.left = window.getComputedStyle(elmtToPosition).left;
        }
        if (posSettings.offsetY) {

            if (typeof posSettings.offsetY === 'function') {
                posSettings.offsetY = posSettings.offsetY(); // function must return a proper number or string
            }

            typeof posSettings.offsetY === 'number'
                ? elmtToPosition.style.top = `calc(${calculatedPosition.top} + ${posSettings.offsetY}px)`
                : elmtToPosition.style.top = `calc(${calculatedPosition.top} + ${posSettings.offsetY})`;
            calculatedPosition.top = window.getComputedStyle(elmtToPosition).top;
        }

        // apply minLeft
        if (posSettings.minLeft) {

            if (typeof posSettings.minLeft === 'function') {
                posSettings.minLeft = posSettings.minLeft(); // function must return a proper number or string
            }

            // save current left of panel as pixel value
            let initialLeft = parseFloat(calculatedPosition.left);
            // convert minLeft number to pixel value
            if (typeof posSettings.minLeft === 'number') {posSettings.minLeft += 'px';}
            // now position panel with minLeft value
            elmtToPosition.style.left = posSettings.minLeft;
            // get changed left as pixel value
            let minLeft = parseFloat(window.getComputedStyle(elmtToPosition).left);
            // compare minLeft with initialLeft and reposition panel again if needed
            if (initialLeft > minLeft) {elmtToPosition.style.left = initialLeft + 'px';}
            calculatedPosition.left = window.getComputedStyle(elmtToPosition).left;
        }
        // apply maxLeft
        if (posSettings.maxLeft) {

            if (typeof posSettings.maxLeft === 'function') {
                posSettings.maxLeft = posSettings.maxLeft(); // function must return a proper number or string
            }

            let initialLeft = parseFloat(calculatedPosition.left);
            if (typeof posSettings.maxLeft === 'number') {posSettings.maxLeft += 'px';}
            elmtToPosition.style.left = posSettings.maxLeft;
            let maxLeft = parseFloat(window.getComputedStyle(elmtToPosition).left);
            if (initialLeft < maxLeft) {elmtToPosition.style.left = initialLeft + 'px';}
            calculatedPosition.left = window.getComputedStyle(elmtToPosition).left;
        }
        // apply maxTop
        if (posSettings.maxTop) {

            if (typeof posSettings.maxTop === 'function') {
                posSettings.maxTop = posSettings.maxTop(); // function must return a proper number or string
            }

            let initialTop = parseFloat(calculatedPosition.top);
            if (typeof posSettings.maxTop === 'number') {posSettings.maxTop += 'px';}
            elmtToPosition.style.top = posSettings.maxTop;
            let maxTop = parseFloat(window.getComputedStyle(elmtToPosition).top);
            if (initialTop < maxTop) {elmtToPosition.style.top = initialTop + 'px';}
            calculatedPosition.top = window.getComputedStyle(elmtToPosition).top;
        }
        // apply minTop
        if (posSettings.minTop) {

            if (typeof posSettings.minTop === 'function') {
                posSettings.minTop = posSettings.minTop(); // function must return a proper number or string
            }

            let initialTop = parseFloat(calculatedPosition.top);
            if (typeof posSettings.minTop === 'number') {posSettings.minTop += 'px';}
            elmtToPosition.style.top = posSettings.minTop;
            let minTop = parseFloat(window.getComputedStyle(elmtToPosition).top);
            if (initialTop > minTop) {elmtToPosition.style.top = initialTop + 'px';}
            calculatedPosition.top = window.getComputedStyle(elmtToPosition).top;
        }

        // apply modify
        if (typeof posSettings.modify === 'function') {
            let modifiedPosition = posSettings.modify.call(calculatedPosition, calculatedPosition);
            elmtToPosition.style.left = modifiedPosition.left;
            elmtToPosition.style.top = modifiedPosition.top;
        }

        elmtToPosition.style.opacity = 1;
        // convert css calc values to pixel values - this is required by dragit and resizeit
        elmtToPosition.style.left = window.getComputedStyle(elmtToPosition).left;
        elmtToPosition.style.top = window.getComputedStyle(elmtToPosition).top;

        return elmtToPosition;
    },

    processCallbacks(panel, arg, someOrEvery = 'some', param) {
        // if arg != array make it one
        if(typeof arg === 'function') {
            arg = [arg];
        }
        // some():  execute callbacks until one is found returning a truthy value
        // every(): execute callbacks until one is found returning a falsy value
        // truthy values are: '0' (string with single zero), 'false' (string with text false), [] (empty array), {} (empzy object), function(){} ("empty" function)
        // falsy values are: false, 0, '', "", null, undefined, NaN
        if (someOrEvery) {
            return arg[someOrEvery](function (cb) {
                if (typeof cb === 'function') {
                    return cb.call(panel, panel, param);
                }
            });
        } else {
            arg.forEach(function (cb) {
                cb.call(panel, panel, param);
            });
        }
    },

    rgbToHsl(r, g, b) {
        r /= 255, g /= 255, b /= 255;
        let max = Math.max(r, g, b),
            min = Math.min(r, g, b),
            h, s, l = (max + min) / 2;
        if (max === min) {
            h = s = 0; // achromatic
        } else {
            let d = max - min;
            s = l > 0.5 ? d / (2 - max - min) : d / (max + min);

            switch (max) {
            case r:
                h = (g - b) / d + (g < b ? 6 : 0);
                break;
            case g:
                h = (b - r) / d + 2;
                break;
            case b:
                h = (r - g) / d + 4;
                break;
            }
            h /= 6;
        }
        //return [ h, s, l ];
        h = h * 360;
        s = s * 100 + '%';
        l = l * 100 + '%';
        return {css: 'hsl(' + h + ',' + s + ',' + l + ')', h: h, s: s, l: l};
    },

    rgbToHex(r, g, b) {
        let red = Number(r).toString(16),
            green = Number(g).toString(16),
            blue = Number(b).toString(16);
        if (red.length === 1) {
            red = `0${red}`;
        }
        if (green.length === 1) {
            green = `0${green}`;
        }
        if (blue.length === 1) {
            blue = `0${blue}`;
        }
        return `#${red}${green}${blue}`;
    },

    removeSnapAreas(panel) {
        document.querySelectorAll('.jsPanel-snap-area').forEach(function (el) {
            if (panel.parentElement) {
                panel.parentElement.removeChild(el);
            }
        });
    },

    resetZi() {
        this.zi = ((startValue = jsPanel.ziBase) => {
            let val = startValue;
            return {next: () => {return val++;}};
        })();
        Array.prototype.slice.call(document.querySelectorAll('.jsPanel-standard'))
            .sort((a, b) => {return a.style.zIndex - b.style.zIndex;})
            .forEach((panel) => {panel.style.zIndex = jsPanel.zi.next();});
    },

    resizeit(elmt, options = {}) {
        let opts            = Object.assign({}, this.defaults.resizeit, options),
            elmtParent        = elmt.parentElement,
            elmtParentTagName = elmtParent.tagName.toLowerCase(),
            maxWidth          = typeof opts.maxWidth === 'function' ? opts.maxWidth() : opts.maxWidth || 10000,
            maxHeight         = typeof opts.maxHeight === 'function' ? opts.maxHeight() : opts.maxHeight || 10000,
            minWidth          = typeof opts.minWidth === 'function' ? opts.minWidth() : opts.minWidth,
            minHeight         = typeof opts.minHeight === 'function' ? opts.minHeight() : opts.minHeight,
            containment,
            resizePanel,
            resizestarted,
            w, h;
        const jspanelresizestart = new CustomEvent('jspanelresizestart', {detail: elmt.id}),
            jspanelresize      = new CustomEvent('jspanelresize', {detail: elmt.id}),
            jspanelresizestop  = new CustomEvent('jspanelresizestop', {detail: elmt.id});


        // normalize containment config
        containment = this.pOcontainment(opts.containment);

        opts.handles.split(',').forEach((item) => {
            const node = document.createElement('DIV');
            node.className = `jsPanel-resizeit-handle jsPanel-resizeit-${item.trim()}`;
            node.style.zIndex = 90;
            elmt.append(node);
        });

        elmt.querySelectorAll('.jsPanel-resizeit-handle').forEach((handle) => {

            jsPanel.pointerdown.forEach(function (item) {
                handle.addEventListener(item, (e) => {
                    // prevent window scroll while resizing elmt
                    e.preventDefault();

                    // disable resizing for all mouse buttons but left
                    if (e.button && e.button > 0) { return false; }

                    elmt.content.style.pointerEvents = 'none';

                    const elmtRect = elmt.getBoundingClientRect(),           /* needs to be calculated on pointerdown!! */
                        elmtParentRect = elmtParent.getBoundingClientRect(), /* needs to be calculated on pointerdown!! */
                        elmtParentStyles = window.getComputedStyle(elmtParent, null),
                        elmtParentBLW = parseInt(elmtParentStyles.borderLeftWidth, 10),
                        elmtParentBTW = parseInt(elmtParentStyles.borderTopWidth, 10),
                        elmtParentPosition = elmtParentStyles.getPropertyValue('position'),
                        startX = e.clientX || e.touches[0].clientX,
                        startY = e.clientY || e.touches[0].clientY,
                        startWidth = elmtRect.width,
                        startHeight = elmtRect.height,
                        resizeHandleClassList = e.target.classList;
                    let startLeft = elmtRect.left,
                        startTop = elmtRect.top,
                        maxWidthEast = 10000, maxWidthWest = 10000, maxHeightSouth = 10000, maxHeightNorth = 10000;

                    if (elmtParentTagName !== 'body') {
                        startLeft = elmtRect.left - elmtParentRect.left + elmtParent.scrollLeft;
                        startTop = elmtRect.top - elmtParentRect.top + elmtParent.scrollTop;
                    }

                    // calc min/max left/top values if containment is set - code from jsDraggable
                    if (elmtParentTagName === 'body' && containment) {
                        maxWidthEast = document.documentElement.clientWidth - elmtRect.left;
                        maxHeightSouth = document.documentElement.clientHeight - elmtRect.top;
                        maxWidthWest = elmtRect.width + elmtRect.left;
                        maxHeightNorth = elmtRect.height + elmtRect.top;

                    } else {
                        // if panel is NOT in body
                        if (containment) {
                            if (elmtParentPosition === 'static') {
                                maxWidthEast = elmtParentRect.width - elmtRect.left + elmtParentBLW;
                                maxHeightSouth = elmtParentRect.height + elmtParentRect.top - elmtRect.top + elmtParentBTW;
                                maxWidthWest = elmtRect.width + (elmtRect.left - elmtParentRect.left) - elmtParentBLW;
                                maxHeightNorth = elmtRect.height + (elmtRect.top - elmtParentRect.top) - elmtParentBTW;
                            } else {
                                maxWidthEast = elmtParent.clientWidth - (elmtRect.left - elmtParentRect.left) + elmtParentBLW;
                                maxHeightSouth = elmtParent.clientHeight - (elmtRect.top - elmtParentRect.top) + elmtParentBTW;
                                maxWidthWest = elmtRect.width + (elmtRect.left - elmtParentRect.left) - elmtParentBLW;
                                maxHeightNorth = elmt.clientHeight + (elmtRect.top - elmtParentRect.top) - elmtParentBTW;
                            }
                        }
                    }
                    // if original opts.containment is array
                    if (containment) {
                        maxWidthWest -= containment[3];
                        maxHeightNorth -= containment[0];
                        maxWidthEast -= containment[1];
                        maxHeightSouth -= containment[2];
                    }

                    // calculate corrections for rotated panels
                    const computedStyle = window.getComputedStyle(elmt);
                    let wDif = parseFloat(computedStyle.width) - elmtRect.width,
                        hDif = parseFloat(computedStyle.height) - elmtRect.height,
                        xDif = parseFloat(computedStyle.left) - elmtRect.left,
                        yDif = parseFloat(computedStyle.top) - elmtRect.top;
                    if (elmtParent !== document.body) {
                        xDif += elmtParentRect.left;
                        yDif += elmtParentRect.top;
                    }

                    resizePanel = (evt) => {
                        // trigger resizestarted only once per resize
                        if (!resizestarted) {
                            document.dispatchEvent(jspanelresizestart);
                            if (opts.start) {
                                jsPanel.processCallbacks(elmt, opts.start, false, {width: startWidth, height: startHeight});
                            }
                            jsPanel.front(elmt);
                        }
                        resizestarted = 1;
                        // trigger resize permanently while resizing
                        document.dispatchEvent(jspanelresize);

                        if (resizeHandleClassList.contains('jsPanel-resizeit-e')
                            || resizeHandleClassList.contains('jsPanel-resizeit-se')
                            || resizeHandleClassList.contains('jsPanel-resizeit-ne')) {

                            w = startWidth + (evt.clientX || evt.touches[0].clientX) - startX + wDif;
                            if (w >= maxWidthEast) {w = maxWidthEast;}
                            if (w >= maxWidth) {w = maxWidth;}
                            else if (w <= minWidth) {w = minWidth;}
                            elmt.style.width = w + 'px';
                        }

                        if (resizeHandleClassList.contains('jsPanel-resizeit-s')
                            || resizeHandleClassList.contains('jsPanel-resizeit-se')
                            || resizeHandleClassList.contains('jsPanel-resizeit-sw')) {

                            h = startHeight + (evt.clientY || evt.touches[0].clientY) - startY + hDif;
                            if (h >= maxHeightSouth) {h = maxHeightSouth;}
                            if (h >= maxHeight) {h = maxHeight;}
                            else if (h <= minHeight) {h = minHeight;}
                            elmt.style.height = h + 'px';
                        }

                        if (resizeHandleClassList.contains('jsPanel-resizeit-w')
                            || resizeHandleClassList.contains('jsPanel-resizeit-nw')
                            || resizeHandleClassList.contains('jsPanel-resizeit-sw')) {

                            w = startWidth + startX - (evt.clientX || evt.touches[0].clientX) + wDif;
                            if (w <= maxWidth && w >= minWidth && w <= maxWidthWest) {elmt.style.left = (startLeft + (evt.clientX || evt.touches[0].clientX) - startX + xDif)+'px';}
                            if (w >= maxWidthWest) {w = maxWidthWest;}
                            if (w >= maxWidth) {w = maxWidth;}
                            else if (w <= minWidth) {w = minWidth;}
                            elmt.style.width = w + 'px';
                        }

                        if (resizeHandleClassList.contains('jsPanel-resizeit-n')
                            || resizeHandleClassList.contains('jsPanel-resizeit-nw')
                            || resizeHandleClassList.contains('jsPanel-resizeit-ne')) {

                            h = startHeight + startY - (evt.clientY || evt.touches[0].clientY) + hDif;
                            if (h <= maxHeight && h >= minHeight && h <= maxHeightNorth) {elmt.style.top = (startTop + (evt.clientY || evt.touches[0].clientY) - startY + yDif)+'px';}
                            if (h >= maxHeightNorth) {h = maxHeightNorth;}
                            if (h >= maxHeight) {h = maxHeight;}
                            else if (h <= minHeight) {h = minHeight;}
                            elmt.style.height = h + 'px';
                        }

                        window.getSelection().removeAllRanges();

                        // get current position and size values while resizing
                        const styles = window.getComputedStyle(elmt),
                            values = {
                                left:   parseFloat(styles.left),
                                top:    parseFloat(styles.top),
                                right:  parseFloat(styles.right),
                                bottom: parseFloat(styles.bottom),
                                width:  parseFloat(styles.width),
                                height: parseFloat(styles.height)
                            };

                        // callback while resizing
                        if (opts.resize) {
                            jsPanel.processCallbacks(elmt, opts.resize, false, values);
                        }
                    };

                    jsPanel.pointermove.forEach(function (item) {
                        document.addEventListener(item, resizePanel, false);
                    });

                    // remove resize handler when mouse leaves browser window (mouseleave doesn't work)
                    window.addEventListener('mouseout', (e) => {
                        if (e.relatedTarget === null) {
                            jsPanel.pointermove.forEach(function (item) {
                                document.removeEventListener(item, resizePanel, false);
                            });
                        }
                    }, false);
                });
            });

        });

        jsPanel.pointerup.forEach(function (item) {
            document.addEventListener(item, (e) => {

                jsPanel.pointermove.forEach(function (item) {
                    document.removeEventListener(item, resizePanel, false);
                });

                if (e.target.classList && e.target.classList.contains('jsPanel-resizeit-handle')) {

                    let isLeftChange, isTopChange;
                    const cl = e.target.className;
                    if (cl.match(/jsPanel-resizeit-nw|jsPanel-resizeit-w|jsPanel-resizeit-sw/i)) {
                        isLeftChange = true;
                    }
                    if (cl.match(/jsPanel-resizeit-nw|jsPanel-resizeit-n|jsPanel-resizeit-ne/i)) {
                        isTopChange = true;
                    }

                    // snap panel to grid (doesn't work that well if inside function resizePanel)
                    if (opts.grid && Array.isArray(opts.grid)) {
                        if (opts.grid.length === 1) {
                            opts.grid[1] = opts.grid[0];
                        }
                        const cw   = parseFloat(elmt.style.width),
                            ch   = parseFloat(elmt.style.height),
                            modW = cw % opts.grid[0],
                            modH = ch % opts.grid[1],
                            cx   = parseFloat(elmt.style.left),
                            cy   = parseFloat(elmt.style.top),
                            modX = cx % opts.grid[0],
                            modY = cy % opts.grid[1];

                        if (modW < opts.grid[0] / 2) {
                            elmt.style.width = (cw - modW) + 'px';
                        } else {
                            elmt.style.width = (cw + (opts.grid[0] - modW)) + 'px';
                        }
                        if (modH < opts.grid[1] / 2) {
                            elmt.style.height = (ch - modH) + 'px';
                        } else {
                            elmt.style.height = (ch + (opts.grid[1] - modH)) + 'px';
                        }

                        if (isLeftChange) {
                            if (modX < opts.grid[0] / 2) {
                                elmt.style.left = (cx - modX) + 'px';
                            } else {
                                elmt.style.left = (cx + (opts.grid[0] - modX)) + 'px';
                            }
                        }
                        if (isTopChange) {
                            if (modY < opts.grid[1] / 2) {
                                elmt.style.top = (cy - modY) + 'px';
                            } else {
                                elmt.style.top = (cy + (opts.grid[1] - modY)) + 'px';
                            }
                        }
                    }

                }

                if (resizestarted) {
                    elmt.content.style.pointerEvents = 'inherit';
                    document.dispatchEvent(jspanelresizestop);
                    resizestarted = undefined;
                    elmt.saveCurrentDimensions();
                    elmt.saveCurrentPosition();
                    if (opts.stop) {
                        jsPanel.processCallbacks(elmt, opts.stop, false, {width: parseFloat(elmt.style.width), height: parseFloat(elmt.style.height)});
                    }
                }

                elmt.content.style.pointerEvents = 'inherit';

            }, false);
        });

        // resizeit is initialized - now disable if set
        if (opts.disable) {
            elmt.querySelectorAll('.jsPanel-resizeit-handle').forEach((handle) => {
                handle.style.pointerEvents = 'none';
            });
        }

        return elmt;
    },

    setClass(elmt, classnames) {
        classnames.split(' ').forEach(item => elmt.classList.add(item));
        return elmt;
    },

    remClass(elmt, classnames) {
        classnames.split(' ').forEach(item => elmt.classList.remove(item));
        return elmt;
    },

    setStyle(elmt, stylesobject) {
        for (let prop in stylesobject) {
            if (stylesobject.hasOwnProperty(prop)) {
                let property = String(prop).replace(/-\w/gi, (match) => {
                    return match.substr(-1).toUpperCase();
                });
                elmt.style[property] = stylesobject[prop];
            }
        }
        return elmt;
    },

    snapPanel(panel, pos) {
        // store panel size before it snaps
        panel.currentData.beforeSnap = {
            width: panel.currentData.width,
            height: panel.currentData.height
        };
        // snap panel
        if (pos && typeof pos === 'function') {
            pos.call(panel, panel, panel.snappableTo);
        } else if (pos !== false) {
            let offsets = [0, 0];
            if (panel.options.dragit.snap.containment) {
                if (panel.options.dragit.containment) {
                    const containment = this.pOcontainment(panel.options.dragit.containment),
                        position    = panel.snappableTo;
                    if (position.startsWith('left')) {
                        offsets[0] = containment[3];
                    } else if (position.startsWith('right')) {
                        offsets[0] = -containment[1];
                    }
                    if (position.endsWith('top')) {
                        offsets[1] = containment[0];
                    } else if (position.endsWith('bottom')) {
                        offsets[1] = -containment[2];
                    }
                }
            }
            panel.reposition(`${panel.snappableTo} ${offsets[0]} ${offsets[1]}`);
            panel.snapped = panel.snappableTo;
        }
    },

    // METHOD CREATING THE PANEL ---------------------------------------------
    create(options = {}, cb) {

        // initialize z-index generator
        if (!jsPanel.zi) {
            jsPanel.zi = ((startValue = jsPanel.ziBase) => {
                let val = startValue;
                return {
                    next: () => {
                        return val++;
                    }
                };
            })();
        }

        let closetimer;
        if (options.config) {
            options = Object.assign({}, this.defaults, options.config, options);
            delete options.config;
        } else {
            options = Object.assign({}, this.defaults, options);
        }
        if (!options.id) {
            options.id = `jsPanel-${jsPanel.idCounter += 1}`;
        } else if (typeof options.id === 'function') {
            options.id = options.id();
        }
        const p = document.getElementById(options.id);
        if (p !== null) {
            // if a panel with passed id already exists, front it and return error object
            if (p.classList.contains('jsPanel')) {p.front();}
            const error = new window.jsPanelError('NO NEW PANEL CREATED!\nAn element with the ID <'+ options.id + '> already exists in the document.');
            try {
                throw error;
            } catch (e) {
                if (cb) {cb.call(e, e);}
            }
            return console.error(error.name+':', error.message);
        }

        // check whether container is valid -> if not return and log error
        const panelContainer = this.pOcontainer(options.container, cb);
        if (panelContainer && panelContainer.message) {
            return console.error(panelContainer.name+':', panelContainer.message);
        }

        // normalize maximizedMargin
        options.maximizedMargin = this.pOcontainment(options.maximizedMargin);

        // normalize drag config
        if (options.dragit) {
            ['start', 'drag', 'stop'].forEach(function (item) {
                if (options.dragit[item]) {
                    if (typeof options.dragit[item] === 'function') {
                        options.dragit[item] = [options.dragit[item]];
                    }
                } else {
                    options.dragit[item] = [];
                }
            });
            if (options.dragit.snap) {
                if (typeof options.dragit.snap === 'object') {
                    options.dragit.snap = Object.assign({}, this.defaultSnapConfig, options.dragit.snap);
                } else {
                    options.dragit.snap = this.defaultSnapConfig;
                }
            }
        }

        // normalize resizeit config
        if (options.resizeit) {
            ['start', 'resize', 'stop'].forEach(function (item) {
                if (options.resizeit[item]) {
                    if (typeof options.resizeit[item] === 'function') {
                        options.resizeit[item] = [options.resizeit[item]];
                    }
                } else {
                    options.resizeit[item] = [];
                }
            });
        }

        // normalize on... callbacks
        ['onbeforeclose', 'onbeforemaximize', 'onbeforeminimize', 'onbeforenormalize', 'onbeforesmallify', 'onbeforeunsmallify',
            'onclosed', 'onfronted', 'onmaximized', 'onminimized', 'onnormalized', 'onsmallified', 'onstatuschange', 'onunsmallified'].forEach(function (item) {
            if (options[item]) {
                if (typeof options[item] === 'function') {
                    options[item] = [options[item]];
                }
            } else {
                options[item] = [];
            }
        });

        /* Compat code ------------------------------------------ */
        if (options.headerRemove) {options.header = false;}
        /* ------------------------------------------------------ */

        let self = options.template ? options.template : this.createPanelTemplate();

        // Properties
        self.options       = options;
        self.status        = 'initialized';
        self.currentData   = {};
        self.header        = self.querySelector('.jsPanel-hdr');                  // complete header section
        self.headerbar     = self.header.querySelector('.jsPanel-headerbar');     // log, title and controls
        self.titlebar      = self.header.querySelector('.jsPanel-titlebar');      // div surrounding title h3
        self.headerlogo    = self.headerbar.querySelector('.jsPanel-headerlogo'); // logo only
        self.headertitle   = self.headerbar.querySelector('.jsPanel-title');      // title h3
        self.controlbar    = self.headerbar.querySelector('.jsPanel-controlbar'); // div surrounding all controls
        self.headertoolbar = self.header.querySelector('.jsPanel-hdr-toolbar');
        self.content       = self.querySelector('.jsPanel-content');
        self.footer        = self.querySelector('.jsPanel-ftr');
        self.snappableTo   = false;
        self.snapped       = false;

        // Events
        const jspanelloaded         = new CustomEvent('jspanelloaded', {'detail': options.id}),
            jspanelbeforeclose      = new CustomEvent('jspanelbeforeclose', {'detail': options.id}),
            jspanelclosed           = new CustomEvent('jspanelclosed', {'detail': options.id}),
            jspanelstatuschange     = new CustomEvent('jspanelstatuschange', {'detail': options.id}),
            jspanelbeforenormalize  = new CustomEvent('jspanelbeforenormalize', {'detail': options.id}),
            jspanelnormalized       = new CustomEvent('jspanelnormalized', {'detail': options.id}),
            jspanelbeforemaximize   = new CustomEvent('jspanelbeforemaximize', {'detail': options.id}),
            jspanelmaximized        = new CustomEvent('jspanelmaximized', {'detail': options.id}),
            jspanelbeforeminimize   = new CustomEvent('jspanelbeforeminimize', {'detail': options.id}),
            jspanelminimized        = new CustomEvent('jspanelminimized', {'detail': options.id}),
            jspanelbeforesmallify   = new CustomEvent('jspanelbeforesmallify', {'detail': options.id}),
            jspanelsmallified       = new CustomEvent('jspanelsmallified', {'detail': options.id}),
            jspanelsmallifiedmax    = new CustomEvent('jspanelsmallifiedmax', {'detail': options.id}),
            jspanelbeforeunsmallify = new CustomEvent('jspanelbeforeunsmallify', {'detail': options.id}),
            jspanelfronted          = new CustomEvent('jspanelfronted', {'detail': options.id});

        // controls handlers
        const hasCloseBtn  = self.querySelector('.jsPanel-btn-close'),
            hasMaxBtn      = self.querySelector('.jsPanel-btn-maximize'),
            hasNormBtn     = self.querySelector('.jsPanel-btn-normalize'),
            hasSmallBtn    = self.querySelector('.jsPanel-btn-smallify'),
            hasSmallrevBtn = self.querySelector('.jsPanel-btn-smallifyrev'),
            hasMinBtn      = self.querySelector('.jsPanel-btn-minimize');

        if (hasCloseBtn) {
            jsPanel.pointerup.forEach((item) => {
                hasCloseBtn.addEventListener(item, (e) => {
                    e.preventDefault();
                    // disable draging for all mouse buttons but left
                    if (e.button && e.button > 0) { return false; }
                    self.close();
                });
            });
        }
        if (hasMaxBtn) {
            jsPanel.pointerup.forEach((item) => {
                hasMaxBtn.addEventListener(item, (e) => {
                    e.preventDefault();
                    // disable draging for all mouse buttons but left
                    if (e.button && e.button > 0) { return false; }
                    self.maximize();
                });
            });
        }
        if (hasNormBtn) {
            jsPanel.pointerup.forEach((item) => {
                hasNormBtn.addEventListener(item, (e) => {
                    e.preventDefault();
                    // disable draging for all mouse buttons but left
                    if (e.button && e.button > 0) { return false; }
                    self.normalize();
                });
            });
        }
        if (hasSmallBtn) {
            jsPanel.pointerup.forEach((item) => {
                hasSmallBtn.addEventListener(item, (e) => {
                    e.preventDefault();
                    // disable draging for all mouse buttons but left
                    if (e.button && e.button > 0) { return false; }
                    self.smallify();
                });
            });
        }
        if (hasSmallrevBtn) {
            jsPanel.pointerup.forEach((item) => {
                hasSmallrevBtn.addEventListener(item, (e) => {
                    e.preventDefault();
                    // disable draging for all mouse buttons but left
                    if (e.button && e.button > 0) { return false; }
                    self.unsmallify();
                });
            });
        }
        if (hasMinBtn) {
            jsPanel.pointerup.forEach((item) => {
                hasMinBtn.addEventListener(item, (e) => {
                    e.preventDefault();
                    // disable draging for all mouse buttons but left
                    if (e.button && e.button > 0) { return false; }
                    self.minimize();
                });
            });
        }

        // import extensions (extensions of the individual panel, not the global object jsPanel)
        let extensions = jsPanel.extensions;
        for (let ext in extensions) {
            if (extensions.hasOwnProperty(ext)) {
                self[ext] = extensions[ext];
            }
        }

        // Methods
        self.addToolbar = (place, tb, callback) => {
            if (place === 'header') {
                place = self.headertoolbar;
            } else if (place === 'footer') {
                place = self.footer;
            }

            if (typeof tb === 'string') {
                place.innerHTML = tb;
            } else if (Array.isArray(tb)) {
                tb.forEach((item) => {
                    if (typeof item === 'string') {
                        place.innerHTML += item;
                    } else {
                        place.append(item);
                    }
                });
            } else if (typeof tb === 'function') {
                const tool = tb.call(self, self);
                if (typeof tool === 'string') {
                    place.innerHTML = tool;
                } else {
                    place.append(tool);
                }
            } else {
                place.append(tb);
            }

            place.classList.add('active');
            if (callback) {callback.call(self, self);}
            return self;
        };

        self.applyBuiltInTheme = (themeDetails) => {
            self.classList.add(`jsPanel-theme-${themeDetails.color}`);  // do not remove theme from jsP
            self.header.classList.add(`jsPanel-theme-${themeDetails.color}`);

            // optionally set theme filling
            if (themeDetails.filling) {
                self.content.style.background = '';
                self.content.classList.add(`jsPanel-content-${themeDetails.filling}`);
            }

            if (!options.headerToolbar) {
                self.content.style.background = '';
                self.content.style.borderTop = `1px solid ${self.headertitle.style.color}`;
            }

            return self;
        };

        self.applyArbitraryTheme = (themeDetails) => {
            self.style.backgroundColor = themeDetails.colors[0];
            self.header.style.backgroundColor = themeDetails.colors[0];
            ['.jsPanel-headerlogo', '.jsPanel-title', '.jsPanel-hdr-toolbar'].forEach((item) => {
                self.querySelector(item).style.color = themeDetails.colors[3];
            }, self);
            self.querySelectorAll('.jsPanel-controlbar .jsPanel-btn').forEach((item) => {
                item.style.color = themeDetails.colors[3];
            });

            if (options.headerToolbar) {
                jsPanel.setStyle(self.headertoolbar, {
                    boxShadow: `0 0 1px ${themeDetails.colors[3]} inset`,
                    width:     'calc(100% + 4px)',
                    marginLeft:'-1px'
                });
            } else {
                self.content.style.borderTop = `1px solid ${themeDetails.colors[3]}`;
            }

            if (themeDetails.filling === 'filled') {
                self.content.style.backgroundColor = themeDetails.colors[0];
                self.content.style.color = themeDetails.colors[3];
            } else if (themeDetails.filling === 'filledlight') {
                self.content.style.backgroundColor = themeDetails.colors[1];
            }

            return self;
        };

        self.applyBootstrapTheme = (themeDetails) => {
            const bsTheme = themeDetails.bstheme,
                bsVersion = $.fn.button.Constructor.VERSION[0];

            if (bsVersion === '4') {
                self.classList.add(`bg-${bsTheme}`);
            } else {
                ['panel', `panel-${bsTheme}`].forEach((item) => {
                    self.classList.add(item);
                });
                self.header.classList.add('panel-heading');
            }

            // added support for material-design-for-bootstrap 4.x colors
            if (themeDetails.bs === 'mdb') {
                let mdbColor = `${bsTheme}-color`;
                if (themeDetails.mdbStyle) {
                    mdbColor = `${mdbColor}-dark`;
                }
                self.classList.add(mdbColor);
            }

            // get primary theme color
            let pColor;
            if (bsVersion === '4') {
                pColor = window.getComputedStyle(self).backgroundColor.replace(/\s+/g, '');
            } else {
                pColor = window.getComputedStyle(self.header).backgroundColor.replace(/\s+/g, '');
            }

            // calc font color for header - needed for iconfonts other than default svg icons
            let colors = jsPanel.calcColors(pColor);
            self.header.style.color = colors[3];

            if (themeDetails.filling) {
                self.setTheme(`${pColor} ${themeDetails.filling}`);
            } else {
                self.setTheme(pColor);
            }

            return self;
        };

        self.applyThemeBorder = (themeDetails) => {
            const bordervalues = options.border.split(' ');
            self.style.borderWidth = bordervalues[0];
            self.style.borderStyle = bordervalues[1];
            self.style.borderColor = bordervalues[2];
            if (!themeDetails.bs) {
                if (jsPanel.themes.indexOf(themeDetails.color) === -1) {
                    // arbitrary themes only (for built-in themes it's taken from the css file)
                    bordervalues[2] ? self.style.borderColor = bordervalues[2] : self.style.borderColor = themeDetails.colors[0];
                }
            } else {
                // bootstrap
                let pColor;
                if (window.getComputedStyle(self.header).backgroundColor  === 'transparent') {
                    pColor = window.getComputedStyle(self).backgroundColor.replace(/\s+/g, '');
                } else {
                    pColor = window.getComputedStyle(self.header).backgroundColor.replace(/\s+/g, '');
                }
                bordervalues[2] ? self.style.borderColor = bordervalues[2] : self.style.borderColor = pColor;
            }
            return self;
        };

        self.autopositionRemaining = () => {
            let autoPos;
            ['left-top-down','left-top-right','center-top-down','right-top-down','right-top-left','left-bottom-up','left-bottom-right','center-bottom-up','right-bottom-up','right-bottom-left'].forEach((item) => {
                if (self.classList.contains(item)) {
                    autoPos = item;
                }
            });
            if (autoPos) {
                options.container.querySelectorAll(`.${autoPos}`).forEach((item) => {
                    item.reposition();
                });
            }
        };

        self.borderRadius = (rad = 5) => {
            let br = typeof rad === 'string' ? rad : rad + 'px',
                hdr = self.header.style,
                cont = self.content.style,
                ftr = self.footer.style;
            // set border-radius of outer div
            self.style.borderRadius = br;
            // set border-radius of either header or content section depending on presence of header
            if (self.querySelector('.jsPanel-hdr')) {
                hdr.borderTopLeftRadius = br;
                hdr.borderTopRightRadius = br;
            } else {
                cont.borderTopLeftRadius = br;
                cont.borderTopRightRadius = br;
            }
            // set border-radius of either footer or content section depending on presence of header
            if (self.querySelector('.jsPanel-ftr.active')) {
                ftr.borderBottomLeftRadius = br;
                ftr.borderBottomRightRadius = br;
            } else {
                cont.borderBottomLeftRadius = br;
                cont.borderBottomRightRadius = br;
            }
            return self;
        };

        self.calcSizeFactors = () => {
            const styles = window.getComputedStyle(self);
            if (options.container === document.body) {
                self.hf = parseFloat(self.style.left) / (document.body.clientWidth - parseFloat(self.style.width));
                self.vf = parseFloat(self.style.top) / (window.innerHeight - parseFloat(styles.height));
            } else {
                let parentStyles = self.parentElement.getBoundingClientRect();
                self.hf = parseFloat(self.style.left) / (parentStyles.width - parseFloat(self.style.width));
                self.vf = parseFloat(self.style.top) / (parentStyles.height - parseFloat(styles.height));
            }
        };

        self.clearTheme = (callback) => {
            jsPanel.themes.concat(jsPanel.mdbthemes).forEach((value) => {
                ['panel', `jsPanel-theme-${value}`, `panel-${value}`, `${value}-color`].forEach((item) => {
                    self.classList.remove(item);
                });
                self.header.classList.remove('panel-heading', `jsPanel-theme-${value}`);
            }, self);
            self.headertitle.classList.remove('panel-title');
            self.content.classList.remove('panel-body', 'jsPanel-content-filled', 'jsPanel-content-filledlight');
            self.footer.classList.remove('panel-footer');
            jsPanel.setStyle(self, {backgroundColor:'', borderWidth:'', borderStyle:'', borderColor:''});
            jsPanel.setStyle(self.content, {background:'', border:''});
            jsPanel.setStyle(self.headertoolbar, {boxShadow:'', width:'', marginLeft:''});
            self.header.style.background = '';
            Array.prototype.slice.call(self.controlbar.querySelectorAll('.jsPanel-icon'))
                .concat([self.headerlogo, self.headertitle, self.headertoolbar, self.content])
                .forEach((item) => {item.style.color = '';});

            if (callback) {callback.call(self, self);}
            return self;
        };

        self.close = (callback) => {

            const doClose = () => {
                const panelId = options.id;

                if (closetimer) {window.clearTimeout(closetimer);}

                self.closeChildpanels();
                if (self.parentElement) {
                    self.parentElement.removeChild(self);
                }
                // return false if panel was not removed from dom
                if (document.querySelector('#'+panelId)) {
                    return false;
                }
                self.removeMinimizedReplacement();
                document.dispatchEvent(jspanelclosed);

                if (callback) {callback.call(panelId, panelId);}

                if (options.onclosed) {
                    jsPanel.processCallbacks(self, options.onclosed, 'every');
                }

                // if panel is autopositioned reposition remaining autopositioned panels
                self.autopositionRemaining();
            };

            document.dispatchEvent(jspanelbeforeclose);

            if (options.onbeforeclose && options.onbeforeclose.length > 0 && !jsPanel.processCallbacks(self, options.onbeforeclose)) {return self;}

            if (options.animateOut) {
                if (options.animateIn) {
                    jsPanel.remClass(self, options.animateIn);
                }
                jsPanel.setClass(self, options.animateOut);
                self.addEventListener('animationend', () => {
                    doClose();
                });
            } else {
                doClose();
            }

        };

        self.closeChildpanels = (callback) => {
            self.getChildpanels().forEach((item) => item.close());
            if (callback) {callback.call(self, self);}
            return self;
        };

        self.contentRemove = (callback) => {
            jsPanel.emptyNode(self.content);
            if (callback) {callback.call(self, self);}
            return self;
        };

        self.createMinimizedReplacement = () => {
            const tpl = jsPanel.createMinimizedTemplate(),
                color = window.getComputedStyle(self.headertitle).color,
                font = options.iconfont,
                controlbar = tpl.querySelector('.jsPanel-controlbar');

            tpl.style.backgroundColor = window.getComputedStyle(self.header).backgroundColor === 'transparent'
                ? window.getComputedStyle(self).backgroundColor
                : window.getComputedStyle(self.header).backgroundColor;
            tpl.id = self.id + '-min';
            tpl.querySelector('.jsPanel-headerbar').replaceChild(self.headerlogo.cloneNode(true), tpl.querySelector('.jsPanel-headerlogo'));
            tpl.querySelector('.jsPanel-titlebar').replaceChild(self.headertitle.cloneNode(true), tpl.querySelector('.jsPanel-title'));
            tpl.querySelector('.jsPanel-title').style.color = color;
            controlbar.style.color = color;

            // set iconfont
            self.setIconfont(font, tpl);

            if (self.dataset.btnnormalize === 'enabled') {
                jsPanel.pointerup.forEach(function (evt) {
                    tpl.querySelector('.jsPanel-btn-normalize').addEventListener(evt, () => {
                        self.normalize();
                    });
                });
            } else {
                controlbar.querySelector('.jsPanel-btn-normalize').style.display = 'none';
            }
            if (self.dataset.btnmaximize === 'enabled') {
                jsPanel.pointerup.forEach(function (evt) {
                    tpl.querySelector('.jsPanel-btn-maximize').addEventListener(evt, () => {
                        self.maximize();
                    });
                });
            } else {
                controlbar.querySelector('.jsPanel-btn-maximize').style.display = 'none';
            }
            if (self.dataset.btnclose === 'enabled') {
                jsPanel.pointerup.forEach(function (evt) {
                    tpl.querySelector('.jsPanel-btn-close').addEventListener(evt, () => {
                        self.close();
                    });
                });
            } else {
                controlbar.querySelector('.jsPanel-btn-close').style.display = 'none';
            }

            return tpl;
        };

        self.dragit = (string) => {
            const dragitOptions = Object.assign({}, jsPanel.defaults.dragit, options.dragit),
                handles = self.querySelectorAll(dragitOptions.handles);
            if (string === 'disable') {
                handles.forEach((handle) => {
                    handle.style.pointerEvents = 'none';
                });
            } else {
                handles.forEach((handle) => {
                    handle.style.pointerEvents = 'auto';
                });
            }
            return self;
        };

        self.front = (callback, execOnFrontedCallbacks = true) => {
            jsPanel.front(self);
            document.dispatchEvent(jspanelfronted);
            if (callback) {callback.call(self, self);}
            if (options.onfronted && execOnFrontedCallbacks) {
                jsPanel.processCallbacks(self, options.onfronted, 'every');
            }
            return self;
        };

        self.getChildpanels = () => {
            return Array.prototype.slice.call(self.content.querySelectorAll('.jsPanel'));
        };

        self.getThemeDetails = (th) => {
            const passedTheme = th.toLowerCase().replace(/ /g, ''),
                theme = {color: false, colors: false, filling: false, bs: false, bstheme: false};

            if (passedTheme.substr(-6, 6) === 'filled') {
                theme.filling = 'filled';
                theme.color = passedTheme.substr(0, passedTheme.length - 6);
            } else if (passedTheme.substr(-11, 11) === 'filledlight') {
                theme.filling = 'filledlight';
                theme.color = passedTheme.substr(0, passedTheme.length - 11);
            } else {
                theme.filling = '';
                theme.color = passedTheme; // themeDetails.color is the primary color
            }
            theme.colors = jsPanel.calcColors(theme.color);

            // if first part of theme includes a "-" it's assumed to be a bootstrap theme
            if (theme.color.match('-')) {
                const bsVariant = theme.color.split('-');
                theme.bs = bsVariant[0];
                theme.bstheme = bsVariant[1];
                theme.mdbStyle = bsVariant[2] || undefined;
            }

            return theme;
        };

        self.isChildpanel = () => {
            // if panel is childpanel of another panel returns parentpanel
            const pp = self.closest('.jsPanel-content');
            return pp ? pp.parentElement : false;
        };

        self.maximize = (callback) => {
            // Note: do not disable maximize method for already maximized panels -> onwindowresize wouldn't work

            if (options.onbeforemaximize && options.onbeforemaximize.length > 0 && !jsPanel.processCallbacks(self, options.onbeforemaximize)) {return self;}

            document.dispatchEvent(jspanelbeforemaximize);

            const parent = self.parentElement,
                margins = options.maximizedMargin;

            if (parent === document.body) {
                // maximize within window
                self.style.width = (document.documentElement.clientWidth - margins[1] - margins[3]) + 'px';
                self.style.height = (document.documentElement.clientHeight - margins[0] - margins[2]) + 'px';
                self.style.left = margins[3] + 'px';
                self.style.top = margins[0] + 'px';

                if (!options.position.fixed) {
                    self.style.left = (window.pageXOffset + margins[3]) + 'px';
                    self.style.top = (window.pageYOffset + margins[0]) + 'px';
                }
            } else {
                // maximize within parentElement
                self.style.width = (parent.clientWidth - margins[1] - margins[3]) + 'px';
                self.style.height = (parent.clientHeight - margins[0] - margins[2]) + 'px';
                self.style.left = margins[3] + 'px';
                self.style.top = margins[0] + 'px';
            }

            self.removeMinimizedReplacement();
            self.status = 'maximized';
            self.setControls(['.jsPanel-btn-maximize', '.jsPanel-btn-smallifyrev']);
            jsPanel.front(self);
            document.dispatchEvent(jspanelmaximized);
            document.dispatchEvent(jspanelstatuschange);

            if (options.onstatuschange) {
                jsPanel.processCallbacks(self, options.onstatuschange, 'every');
            }

            if (callback) {callback.call(self, self);}

            if (options.onmaximized) {
                jsPanel.processCallbacks(self, options.onmaximized, 'every');
            }

            return self;
        };

        self.minimize = (callback) => {
            if  (self.status === 'minimized') {
                return self;
            }

            if (options.onbeforeminimize && options.onbeforeminimize.length > 0 && !jsPanel.processCallbacks(self, options.onbeforeminimize)) {return self;}

            document.dispatchEvent(jspanelbeforeminimize);

            // create container for minimized replacements if not already there
            if (!document.getElementById('jsPanel-replacement-container')) {
                const replacementContainer = document.createElement('div');
                replacementContainer.id = 'jsPanel-replacement-container';
                document.body.append(replacementContainer);
            }

            self.style.left = '-9999px';
            self.statusBefore = self.status;
            self.status = 'minimized';
            document.dispatchEvent(jspanelminimized);
            document.dispatchEvent(jspanelstatuschange);

            if (options.onstatuschange) {
                jsPanel.processCallbacks(self, options.onstatuschange, 'every');
            }

            if (options.minimizeTo) {
                const replacement = self.createMinimizedReplacement();
                let container, parent, list;
                if (options.minimizeTo === 'default') {
                    document.getElementById('jsPanel-replacement-container').append(replacement);
                } else if (options.minimizeTo === 'parentpanel') {
                    parent = self.closest('.jsPanel-content').parentElement;
                    list = parent.querySelectorAll('.jsPanel-minimized-box');
                    container = list[list.length - 1];
                    container.append(replacement);
                } else if (options.minimizeTo === 'parent') {
                    parent = self.parentElement;
                    container = parent.querySelector('.jsPanel-minimized-container');
                    if (!container) {
                        container = document.createElement('div');
                        container.className = 'jsPanel-minimized-container';
                        parent.append(container);
                    }
                    container.append(replacement);
                } else {
                    // all other strings are assumed to be selector strings returning a single element to append the min replacement to
                    document.querySelector(options.minimizeTo).append(replacement);
                }
            }

            if (callback) {callback.call(self, self);}

            if (options.onminimized) {
                jsPanel.processCallbacks(self, options.onminimized, 'every');
            }

            return self;
        };

        self.normalize = (callback) => {
            if  (self.status === 'normalized') {
                return self;
            }

            if (options.onbeforenormalize && options.onbeforenormalize.length > 0 && !jsPanel.processCallbacks(self, options.onbeforenormalize)) {return self;}

            document.dispatchEvent(jspanelbeforenormalize);
            self.style.width = self.currentData.width;
            self.style.height = self.currentData.height;
            self.style.left = self.currentData.left;
            self.style.top = self.currentData.top;
            self.removeMinimizedReplacement();
            self.status = 'normalized';
            self.setControls(['.jsPanel-btn-normalize', '.jsPanel-btn-smallifyrev']);
            jsPanel.front(self);
            document.dispatchEvent(jspanelnormalized);
            document.dispatchEvent(jspanelstatuschange);

            if (options.onstatuschange) {
                jsPanel.processCallbacks(self, options.onstatuschange, 'every');
            }

            if (callback) {callback.call(self, self);}

            if (options.onnormalized) {
                jsPanel.processCallbacks(self, options.onnormalized, 'every');
            }

            return self;
        };

        self.removeMinimizedReplacement = () => {
            const elmt = document.getElementById(self.id + '-min');
            if (elmt) {
                elmt.parentElement.removeChild(elmt);
            }
            return self;
        };

        self.reposition = (...params) => {
            let pos = options.position,
                updateCache = true,
                callback;
            params.forEach(function (value) {
                if (typeof value === 'string' || typeof value === 'object') {
                    pos = value;
                } else if (typeof value === 'boolean') {
                    updateCache = value;
                } else if (typeof value === 'function') {
                    callback = value;
                }
            });

            jsPanel.position(self, pos);
            if (updateCache) {self.saveCurrentPosition();}
            if (callback) {callback.call(self, self);}
            return self;
        };

        self.repositionOnSnap = (pos) => {
            let offsetX = '0', offsetY = '0';
            const margins = jsPanel.pOcontainment(options.dragit.containment);
            // calculate offsets
            if (options.dragit.snap.containment) {
                if (pos === 'left-top') {
                    offsetX = margins[3];
                    offsetY = margins[0];
                } else if (pos === 'right-top') {
                    offsetX = -margins[1];
                    offsetY = margins[0];
                } else if (pos === 'right-bottom') {
                    offsetX = -margins[1];
                    offsetY = -margins[2];
                } else if (pos === 'left-bottom') {
                    offsetX = margins[3];
                    offsetY = -margins[2];
                } else if (pos === 'center-top') {
                    offsetX = margins[3]/2 - margins[1]/2;
                    offsetY = margins[0];
                } else if (pos === 'center-bottom') {
                    offsetX = margins[3]/2 - margins[1]/2;
                    offsetY = -margins[2];
                } else if (pos === 'left-center') {
                    offsetX = margins[3];
                    offsetY = margins[0]/2 - margins[2]/2;
                } else if (pos === 'right-center') {
                    offsetX = -margins[1];
                    offsetY = margins[0]/2 - margins[2]/2;
                }
            }
            /* jsPanel.position(self, `${pos} ${offsetX} ${offsetY}`);
               For some reason I could not find the line above does not work (pos and offsets in one string), but only when
               center-bottom is used with different settings for left/right margin.
            */
            jsPanel.position(self, pos);
            jsPanel.setStyle(self, {
                left: `calc(${self.style.left} + ${offsetX}px)`,
                top:  `calc(${self.style.top} + ${offsetY}px)`
            });
        };

        self.resize = (...params) => {
            const dimensions = window.getComputedStyle(self);
            let size = {width: dimensions.width, height: dimensions.height},
                updateCache = true,
                callback;
            params.forEach(function (value) {
                if (typeof value === 'string') {
                    size = value;
                } else if (typeof value === 'object') {
                    size = Object.assign(size, value);
                } else if (typeof value === 'boolean') {
                    updateCache = value;
                } else if (typeof value === 'function') {
                    callback = value;
                }
            });

            const values = jsPanel.pOsize(self, size);
            self.style.width = values.width;
            self.style.height = values.height;
            if (updateCache) {self.saveCurrentDimensions();}
            if (callback) {callback.call(self, self);}
            return self;
        };

        self.resizeit = (string) => {
            const handles = self.querySelectorAll('.jsPanel-resizeit-handle');
            if (string === 'disable') {
                handles.forEach((handle) => {
                    handle.style.pointerEvents = 'none';
                });
            } else {
                handles.forEach((handle) => {
                    handle.style.pointerEvents = 'auto';
                });
            }
            return self;
        };

        self.saveCurrentDimensions = () => {
            const normData = window.getComputedStyle(self);
            self.currentData.width = normData.width;
            if (self.status === 'normalized') {
                self.currentData.height = normData.height;
            }
        };
        self.saveCurrentPosition = () => {
            const normData = window.getComputedStyle(self);
            self.currentData.left = normData.left;
            self.currentData.top = normData.top;
        };

        self.setControls = (sel, callback) => {
            self.header.querySelectorAll('.jsPanel-btn').forEach((item) => {
                item.style.display = 'block';
            });
            sel.forEach((item) => {
                const btn = self.controlbar.querySelector(item);
                if (btn) {
                    btn.style.display = 'none';
                }
            });
            if (callback) {callback.call(self, self);}
            return self;
        };

        self.setControlStatus = (ctrl, action = 'enable', callback) => {
            if (action === 'disable') {
                if (self.getAttribute(`data-btn${ctrl}`) !== 'removed') {
                    self.setAttribute(`data-btn${ctrl}`, 'disabled');
                    const btn = self.controlbar.querySelector(`.jsPanel-btn-${ctrl}`);
                    btn.style.pointerEvents = 'none';
                    btn.style.opacity = 0.4;
                    btn.style.cursor = 'default';
                }
            } else if (action === 'enable') {
                if (self.getAttribute(`data-btn${ctrl}`) !== 'removed') {
                    self.setAttribute(`data-btn${ctrl}`, 'enabled');
                    const btn = self.controlbar.querySelector(`.jsPanel-btn-${ctrl}`);
                    btn.style.pointerEvents = 'auto';
                    btn.style.opacity = 1;
                    btn.style.cursor = 'pointer';
                }
            } else if (action === 'remove') {
                const btn = self.controlbar.querySelector(`.jsPanel-btn-${ctrl}`);
                self.controlbar.removeChild(btn);
                self.setAttribute(`data-btn${ctrl}`, 'removed');
            }

            if (callback) {callback.call(self, self);}
            return self;
        };

        self.setHeaderControls = (callback) => {
            const controls = ['close', 'maximize', 'normalize', 'minimize', 'smallify', 'smallifyrev'],
                option = options.headerControls;
            if (typeof option === 'string') {
                if (option === 'none') {
                    controls.forEach((item) => {
                        self.setControlStatus(item, 'remove');
                    });
                } else if (option === 'closeonly') {
                    controls.forEach((item) => {
                        if (item !== 'close') {self.setControlStatus(item, 'remove');}
                    });
                }
            } else {
                controls.forEach((item) => {
                    if (option[item]) {
                        self.setControlStatus(item, option[item]);
                    }
                });
            }
            if (callback) {callback.call(self, self);}
            return self;
        };

        self.setHeaderLogo = (hdrLogo, callback) => {
            let logos = [self.headerlogo],
                minPanel = document.querySelector('#' + self.id + '-min');
            if (minPanel) {
                logos.push(minPanel.querySelector('.jsPanel-headerlogo'));
            }

            if (typeof hdrLogo === 'string') {
                if (hdrLogo.substr(0,1) !== '<') {
                    // is assumed to be an img url
                    logos.forEach(function (item) {
                        jsPanel.emptyNode(item);
                        const img = document.createElement('img');
                        img.src = hdrLogo;
                        item.append(img);
                    });
                } else {
                    logos.forEach(function (item) {
                        item.innerHTML = hdrLogo;
                    });
                }
            } else {
                // assumed to be a node object
                logos.forEach(function (item) {
                    jsPanel.emptyNode(item);
                    item.append(hdrLogo);
                });
            }
            // set max-height of logo to equal height of headerbar
            logos.forEach(function (item) {
                item.querySelectorAll('img').forEach(function (img) {
                    img.style.maxHeight = getComputedStyle(self.headerbar).height;
                });
            });

            if (callback) {callback.call(self, self);}
            return self;
        };

        self.setHeaderRemove = (callback) => {
            self.removeChild(self.header);
            self.content.classList.add('jsPanel-content-noheader');
            ['close', 'maximize', 'normalize', 'minimize', 'smallify', 'smallifyrev'].forEach((item) => {
                self.setAttribute(`data-btn${item}`, 'removed');
            });

            if (callback) {callback.call(self, self);}
            return self;
        };

        self.setHeaderTitle = (hdrTitle, callback) => {
            let titles = [self.headertitle],
                minPanel = document.querySelector('#' + self.id + '-min');
            if (minPanel) {
                titles.push(minPanel.querySelector('.jsPanel-title'));
            }
            if (typeof hdrTitle === 'string') {
                titles.forEach(function (item) {
                    item.innerHTML = hdrTitle;
                });
            } else if (typeof hdrTitle === 'function') {
                titles.forEach(function (item) {
                    jsPanel.emptyNode(item);
                    item.innerHTML = hdrTitle();
                });
            } else {
                // assumed to be a node object
                titles.forEach(function (item) {
                    jsPanel.emptyNode(item);
                    item.append(hdrTitle);
                });
            }

            if (callback) {callback.call(self, self);}
            return self;
        };

        self.setIconfont = (font = false, panel = self, callback) => {
            if (font !== false) {
                let classArray, textArray;
                if (font === 'bootstrap' || font === 'glyphicon') {
                    classArray = ['glyphicon glyphicon-remove', 'glyphicon glyphicon-fullscreen', 'glyphicon glyphicon-resize-full', 'glyphicon glyphicon-minus', 'glyphicon glyphicon-chevron-down', 'glyphicon glyphicon-chevron-up'];
                } else if (font === 'fa' || font === 'far' || font === 'fal' || font === 'fas') {
                    classArray = [`${font} fa-window-close`, `${font} fa-window-maximize`, `${font} fa-window-restore`, `${font} fa-window-minimize`, `${font} fa-chevron-down`, `${font} fa-chevron-up`];
                } else if (font === 'material-icons') {
                    classArray = [font, font, font, font, font, font];
                    textArray = ['close', 'fullscreen', 'fullscreen_exit', 'call_received', 'expand_more', 'expand_less'];
                    panel.controlbar.querySelectorAll('.jsPanel-btn').forEach(function (item) {
                        item.style.padding = '6px 0 8px 0';
                    });
                } else if (Array.isArray(font)) {
                    classArray = [`custom-control-icon ${font[5]}`, `custom-control-icon ${font[4]}`, `custom-control-icon ${font[3]}`, `custom-control-icon ${font[2]}`, `custom-control-icon ${font[1]}`, `custom-control-icon ${font[0]}`];
                } else {
                    return panel;
                }
                panel.querySelectorAll('.jsPanel-controlbar .jsPanel-btn').forEach(item => {
                    jsPanel.emptyNode(item).innerHTML = '<span></span>';
                });
                Array.prototype.slice.call(panel.querySelectorAll('.jsPanel-controlbar .jsPanel-btn > span')).reverse().forEach((item, i) => {
                    item.className = classArray[i];
                    if (font === 'material-icons') {
                        item.textContent = textArray[i];
                    }
                });
            }
            if (callback) {callback.call(panel, panel);}
            return panel;
        };

        self.setRtl = () => {
            [self.header, self.headerbar, self.titlebar, self.controlbar, self.headertoolbar, self.footer].forEach(item => {
                item.classList.add('jsPanel-rtl');
            });
            [self.headertitle, self.headertoolbar, self.content, self.footer].forEach(item => {
                item.dir = 'rtl';
                if (options.rtl.lang) {item.lang = options.rtl.lang;}
            });
        };

        self.setSize = () => {
            if (options.panelSize) {
                const values = jsPanel.pOsize(self, options.panelSize);
                self.style.width = values.width;
                self.style.height = values.height;
            } else if (options.contentSize) {
                const values = jsPanel.pOsize(self, options.contentSize);
                self.content.style.width = values.width;
                self.content.style.height = values.height;
                self.style.width = values.width; // explicitly assign current width/height to panel
                self.content.style.width = '100%';
            }
            return self;
        };

        self.setTheme = (theme = options.theme, callback) => {
            // first remove all theme related syles
            self.clearTheme();

            if (theme === 'none') {
                // results in an all white panel without any theme related classes/styles applied
                // removal of footer background/border is done in jsP.toolbarAdd()
                self.style.backgroundColor = '#fff';
                return self;
            }

            const themeDetails = self.getThemeDetails(theme);

            if (!themeDetails.bs) {
                if (jsPanel.themes.indexOf(themeDetails.color) !== -1) {
                    self.applyBuiltInTheme(themeDetails);
                } else {
                    self.applyArbitraryTheme(themeDetails);
                }
            } else {
                self.applyBootstrapTheme(themeDetails);
            }

            if (options.border) {
                self.applyThemeBorder(themeDetails);
            } else {
                self.style.borderWidth = '';
                self.style.borderStyle = '';
                self.style.borderColor = '';
            }

            if (callback) {callback.call(self, self);}
            return self;
        };

        self.smallify = (callback) => {
            if  (self.status === 'smallified' || self.status === 'smallifiedmax') {
                return self;
            }

            if (options.onbeforesmallify && options.onbeforesmallify.length > 0 && !jsPanel.processCallbacks(self, options.onbeforesmallify)) {return self;}

            document.dispatchEvent(jspanelbeforesmallify);

            if (self.status === 'normalized') {
                self.saveCurrentDimensions();
            }

            self.style.overflow = 'hidden';
            let selfStyles = window.getComputedStyle(self),
                selfHeaderHeight = parseFloat(window.getComputedStyle(self.headerbar).height);
            self.style.height = (parseFloat(selfStyles.borderTopWidth) + parseFloat(selfStyles.borderBottomWidth) + selfHeaderHeight) + 'px';

            if (self.status === 'normalized') {
                self.setControls(['.jsPanel-btn-normalize', '.jsPanel-btn-smallify']);
                self.status = 'smallified';
                document.dispatchEvent(jspanelsmallified);
                document.dispatchEvent(jspanelstatuschange);
                if (options.onstatuschange) {
                    jsPanel.processCallbacks(self, options.onstatuschange, 'every');
                }
            } else if (self.status === 'maximized') {
                self.setControls(['.jsPanel-btn-maximize', '.jsPanel-btn-smallify']);
                self.status = 'smallifiedmax';
                document.dispatchEvent(jspanelsmallifiedmax);
                document.dispatchEvent(jspanelstatuschange);
                if (options.onstatuschange) {
                    jsPanel.processCallbacks(self, options.onstatuschange, 'every');
                }
            }

            let minBoxes = self.querySelectorAll('.jsPanel-minimized-box');
            minBoxes[minBoxes.length - 1].style.display = 'none';

            if (callback) {callback.call(self, self);}

            if (options.onsmallified) {
                jsPanel.processCallbacks(self, options.onsmallified, 'every');
            }

            return self;
        };

        self.unsmallify = (callback) => {
            if (self.status === 'smallified' || self.status === 'smallifiedmax') {

                if (options.onbeforeunsmallify && options.onbeforeunsmallify.length > 0 && !jsPanel.processCallbacks(self, options.onbeforeunsmallify)) {return self;}

                document.dispatchEvent(jspanelbeforeunsmallify);
                self.style.overflow = 'visible';
                jsPanel.front(self);

                if (self.status === 'smallified') {
                    self.style.height = self.currentData.height;
                    self.setControls(['.jsPanel-btn-normalize', '.jsPanel-btn-smallifyrev']);
                    self.status = 'normalized';
                    document.dispatchEvent(jspanelnormalized);
                    document.dispatchEvent(jspanelstatuschange);
                    if (options.onstatuschange) {
                        jsPanel.processCallbacks(self, options.onstatuschange, 'every');
                    }
                } else if (self.status === 'smallifiedmax') {
                    self.maximize();
                } else if (self.status === 'minimized') {
                    self.normalize();
                }

                let minBoxes = self.querySelectorAll('.jsPanel-minimized-box');
                minBoxes[minBoxes.length - 1].style.display = 'flex';

                if (callback) {callback.call(self, self);}

                if (options.onunsmallified) {
                    jsPanel.processCallbacks(self, options.onunsmallified, 'every');
                }
            }

            return self;
        };

        // option.id
        self.id = options.id;

        // option.paneltype classname
        self.classList.add('jsPanel-' + options.paneltype);

        // set z-index and paneltype class
        if (options.paneltype === 'standard') {
            self.style.zIndex = this.zi.next();
        }

        // option.container
        panelContainer.append(self);
        self.front(false, false); // just to ensure iframe code in jsPanel.front() works for very first panel as well, second false prevents onfronted callbacks to be executed

        // option.theme
        self.setTheme(options.theme);

        // option.boxShadow
        if (options.boxShadow) {
            self.classList.add(`jsPanel-depth-${options.boxShadow}`);
        }

        /* option.header,
         option.iconfont,
         option.headerControls,
         option.headerLogo,
         option.headerTitle
         */
        if (options.header) {
            if (options.headerLogo) {self.setHeaderLogo(options.headerLogo);}
            self.setIconfont(options.iconfont);
            self.setHeaderTitle(options.headerTitle);
            self.setHeaderControls();

            if (options.header === 'auto-show-hide') {
                let bg = options.theme.split('-'),
                    boxShadow = 'jsPanel-depth-' + options.boxShadow,
                    bgClass = 'bg-',
                    mdbColorClass;
                if (bg[1]) {bgClass += bg[1];}
                if (bg[2]) {mdbColorClass = bg[1] + '-' + 'color-' + bg[2];}

                self.header.style.opacity = 0;

                if (bg[0] === 'bootstrap' || bg[0] === 'mdb') {
                    this.remClass(self, bgClass);
                    if (bg[0] === 'mdb') {
                        this.remClass(self, mdbColorClass);
                    }
                }
                self.style.backgroundColor = 'transparent';
                this.remClass(self, boxShadow);
                this.setClass(self.content, boxShadow);

                self.header.addEventListener('mouseenter', function () {
                    self.header.style.opacity = 1;
                    if (bg[0] === 'bootstrap' || bg[0] === 'mdb') {
                        jsPanel.setClass(self, bgClass);
                        if (bg[0] === 'mdb') {
                            jsPanel.setClass(self, mdbColorClass);
                        }
                    }
                    jsPanel.setClass(self, boxShadow);
                    jsPanel.remClass(self.content, boxShadow);
                });
                self.header.addEventListener('mouseleave', function () {
                    self.header.style.opacity = 0;
                    if (bg[0] === 'bootstrap' || bg[0] === 'mdb') {
                        jsPanel.remClass(self, bgClass);
                        if (bg[0] === 'mdb') {
                            jsPanel.remClass(self, mdbColorClass);
                        }
                    }
                    jsPanel.remClass(self, boxShadow);
                    jsPanel.setClass(self.content, boxShadow);
                });
            }
        } else {
            self.setHeaderRemove();
        }

        // option.headerToolbar
        if (options.headerToolbar) {
            self.addToolbar(self.headertoolbar, options.headerToolbar);
        }
        // option.footerToolbar
        if (options.footerToolbar) {
            self.addToolbar(self.footer, options.footerToolbar);
        }

        // option.borderRadius
        if (options.borderRadius) {
            self.borderRadius(options.borderRadius);
        }

        // option.content
        if (options.content) {
            if (typeof options.content === 'function') {
                options.content.call(self, self);
            } else if (typeof options.content === 'string') {
                self.content.innerHTML = options.content;
            } else {
                self.content.append(options.content);
            }
        }

        // option.contentAjax
        if (options.contentAjax) {
            this.ajax(self, options.contentAjax);
        }

        // option.contentFetch
        if (options.contentFetch) {
            this.fetch(self);
        }

        // option.contentOverflow
        if (options.contentOverflow) {
            let value = options.contentOverflow.split(' ');
            if (value.length === 1) {
                self.content.style.overflow = value[0];
            } else if (value.length === 2) {
                self.content.style.overflowX = value[0];
                self.content.style.overflowY = value[1];
            }
        }

        // option.rtl
        if (options.rtl) {self.setRtl();}

        // option.size -- should be after option.theme
        self.setSize();

        // option.position
        self.status = 'normalized';
        // if option.position evaluates to false panel will not be positioned at all
        if (options.position || options.position !== 'cursor') {
            this.position(self, options.position);
        } else {
            self.style.opacity = 1;
        }
        document.dispatchEvent(jspanelnormalized);
        self.calcSizeFactors();

        // option.animateIn
        if (options.animateIn) {
            // remove class again on animationend, otherwise opacity doesn't change when panel is dragged
            self.addEventListener('animationend', () => {
                this.remClass(self, options.animateIn);
            });
            this.setClass(self, options.animateIn);
        }

        // option.dragit AND option.resizeit AND option.syncMargins
        if (options.syncMargins) {
            const containment = this.pOcontainment(options.maximizedMargin);
            if (options.dragit) {
                options.dragit.containment = containment;
                if (options.dragit.snap) {
                    options.dragit.snap.containment = true;
                }
            }
            if (options.resizeit) {
                options.resizeit.containment = containment;
            }
        }

        if (options.dragit) {
            this.dragit(self, options.dragit);
            self.addEventListener('jspaneldragstop', (e) => {
                if (e.detail === self.id) {
                    self.calcSizeFactors();
                }
            },false);
        } else {
            self.titlebar.style.cursor = 'default';
        }

        if (options.resizeit) {
            this.resizeit(self, options.resizeit);
            let startstatus;
            self.addEventListener('jspanelresizestart', (e) => {
                if (e.detail === self.id) {
                    startstatus = self.status;
                }
            },false);
            self.addEventListener('jspanelresizestop', (e) => {
                if (e.detail === self.id) {
                    if ((startstatus === 'smallified' || startstatus === 'smallifiedmax' || startstatus === 'maximized') && parseFloat(self.style.height) > parseFloat(window.getComputedStyle(self.header).height)) {
                        self.setControls(['.jsPanel-btn-normalize', '.jsPanel-btn-smallifyrev']);
                        self.status = 'normalized';
                        document.dispatchEvent(jspanelnormalized);
                        document.dispatchEvent(jspanelstatuschange);
                        if (options.onstatuschange) {
                            jsPanel.processCallbacks(self, options.onstatuschange, 'every');
                        }
                        self.calcSizeFactors();
                    }
                }
            },false);
        }

        // initialize self.currentData - must be after options position & size
        self.saveCurrentDimensions();
        self.saveCurrentPosition();

        // option.setStatus
        if (options.setStatus) {
            const newStatus = options.setStatus;
            if (newStatus === 'smallifiedmax') {
                self.maximize().smallify();
            } else if (newStatus === 'smallified') {
                self.smallify();
            } else {
                const func = newStatus.substr(0,newStatus.length-1);
                self[func]();
            }
        }

        // option.autoclose
        if (options.autoclose) {
            closetimer = window.setTimeout(() => {if (self) self.close();}, options.autoclose);
        }

        // front panel on mousedown
        this.pointerdown.forEach((item) => {
            self.addEventListener(item, (e) => {
                if (!e.target.closest('.jsPanel-btn-close') && !e.target.closest('.jsPanel-btn-minimize') && options.paneltype === 'standard') {
                    self.front();
                }
            }, false);
        });

        // option.onwindowresize
        if (options.onwindowresize) {
            window.addEventListener('resize', (e) => {
                if (e.target === window) {       // see https://bugs.jqueryui.com/ticket/7514
                    const param = options.onwindowresize,
                        status = self.status,
                        parentStyles = window.getComputedStyle(self.parentElement);
                    if (status === 'maximized' && param === true) {
                        self.maximize();
                    } else if (status === 'normalized' || status === 'smallified' || status === 'maximized') {
                        if (typeof param === 'function') {
                            param.call(self, e, self);
                        } else {
                            self.style.left = (() => {
                                let l;
                                if (options.container === document.body) {
                                    l = (document.body.clientWidth - parseFloat(self.style.width)) * self.hf;
                                } else {
                                    l = (parseFloat(parentStyles.width) - parseFloat(self.style.width)) * self.hf;
                                }
                                return l <= 0 ? 0 : l + 'px';
                            })();
                            self.style.top = (() => {
                                let t;
                                if (options.container === document.body) {
                                    t = (window.innerHeight - parseFloat(self.currentData.height)) * self.vf;
                                } else {
                                    t = (parseFloat(parentStyles.height) - parseFloat(self.currentData.height)) * self.vf;
                                }
                                return t <= 0 ? 0 : t + 'px';
                            })();

                        }
                    }
                }
            },false);
        }

        // without this handler content section would have pointerEvents = none when clicking header section (see dragit)
        this.pointerup.forEach((item) => {
            self.addEventListener(item, () => {
                self.content.style.pointerEvents = 'inherit';
            });
        });

        // global callbacks
        if (this.globalCallbacks) {
            if (Array.isArray(this.globalCallbacks)) {
                this.globalCallbacks.forEach((item) => {
                    item.call(self, self);
                });
            } else {
                this.globalCallbacks.call(self, self);
            }
        }

        // option.callback
        if (options.callback) {
            if (Array.isArray(options.callback)) {
                options.callback.forEach((item) => {
                    item.call(self, self);
                });
            } else {
                options.callback.call(self, self);
            }
        }

        // construtor callback
        if (cb) {cb.call(self, self);}

        document.dispatchEvent(jspanelloaded);
        return self;

    }

};
